"""@package shockpolar
  Plot shock polar curves
"""

import numpy as np
import aerokit.common.defaultgas as defg # relative import is deprecated by doctest
import aerokit.aero.degree      as deg
import aerokit.aero.ShockWave   as sw

import matplotlib.pyplot as plt
from aerokit.aero.plot.defaultstyle import *


def plot_theta_sigma(mach, gamma=defg._gamma, npts=100, curve='both', devmax=False, sonic=False, color='k', linestyle='-', **kwargs):
    """
    	Plot shock polar curve in deviation / shock angle axes

		Long comment
 
		:param mach:       upstream Mach number
        :param gamma:      specific heat ratio, default from aerokit.common.defaultgas
        :param npts:       number of computed points, curve accuracy
        :param curve:      choose which curve to plot (left, right or both)
		:return:     
 
 		:Example:

		.. seealso:: 
		.. note:: 
    """	
 
    sig = np.linspace(deg.asin(1./mach), 90., npts+1)
    dev = sw.deflection_Mach_sigma(mach, sig, gamma)
    if curve in ['right', 'both']: plt.plot( dev, sig, color=color, linestyle=linestyle, **kwargs)
    if curve in ['left',  'both']: plt.plot(-dev, sig, color=color, linestyle=linestyle, **kwargs)
    if devmax:
    	thet = sw.dev_Max(mach, gamma=gamma)
    	sig  = sw.sigma_DevMax(mach, gamma=gamma)
    	if curve in ['right', 'both']: plt.plot( thet, sig, 'ro', alpha=0.9)
    	if curve in ['left', 'both']:  plt.plot(-thet, sig, 'ro', alpha=0.9)
    if sonic:
    	thet = sw.dev_Sonic(mach, gamma=gamma)
    	sig  = sw.sigma_Sonic(mach, gamma=gamma)
    	if curve in ['right', 'both']: plt.plot( thet, sig, 'ko', markerfacecolor='white')
    	if curve in ['left', 'both']:  plt.plot(-thet, sig, 'ko', markerfacecolor='white')


def plot_theta_pressure(mach, gamma=defg._gamma, npts=100, thet_init=0., p_init=1., curve='both', devmax=False, sonic=False, color='k', linestyle='-', ax=plt, **kwargs):
    """
    	Plot shock polar curve in deviation / pressure ratio axes

		Long comment
 
		:param mach:       upstream Mach number
        :param gamma:      specific heat ratio, default from aerokit.common.defaultgas
        :param npts:       number of computed points, curve accuracy
        :param thet_init:  upstream angle (shift the curve by this angle), default 0.
        :param p_init:     reference pressure (shift the curve by this ratio), default 1.
        :param curve:      choose which curve to plot (left, right or both)
		:return:     
 
 		:Example:

		.. seealso:: 
		.. note:: 
    """	
 
    sig = np.linspace(deg.asin(1./mach), 90., npts+1)
    dev = sw.deflection_Mach_sigma(mach, sig, gamma)
    ps  = p_init * sw.Ps_ratio(mach*deg.sin(sig), gamma)     # pressure ratio only depends on normal Mach number
    if curve in ['right', 'both']: ax.plot(thet_init+dev, ps, color=color, linestyle=linestyle, **kwargs)
    if curve in ['left',  'both']: ax.plot(thet_init-dev, ps, color=color, linestyle=linestyle, **kwargs)
    if devmax:
    	thet = sw.dev_Max(mach, gamma=gamma)
    	sig  = sw.sigma_DevMax(mach, gamma=gamma)
    	ps   = sw.Ps_ratio(mach*deg.sin(sig), gamma=gamma)
    	if curve in ['right', 'both']: ax.plot(thet_init+thet, p_init*ps, 'ro', alpha=0.9)
    	if curve in ['left', 'both']:  ax.plot(thet_init-thet, p_init*ps, 'ro', alpha=0.9)
    if sonic:
    	thet = sw.dev_Sonic(mach, gamma=gamma)
    	sig  = sw.sigma_Sonic(mach, gamma=gamma)
    	ps   = sw.Ps_ratio(mach*deg.sin(sig), gamma=gamma)
    	if curve in ['right', 'both']: ax.plot(thet_init+thet, p_init*ps,  'ko', markerfacecolor='white')
    	if curve in ['left', 'both']:  ax.plot(thet_init-thet, p_init*ps,  'ko', markerfacecolor='white')





import numpy as np
import inspect

@np.vectorize
def secant_solve(f, y, x, tol=0.00001):
    """
    secant_solve(f, y, x, tol=0.00001)
      secant method (approximate newton) searching for x with f(x)=y
    """
    itmax = 20
    it    = 1
    old_x = x
    old_y = f(old_x)
    new_x = x*0.999 
    #while (it < itmax) and np.any(abs(new_x-old_x) > np.maximum(tol,tol*abs(new_x))):
    while (it < itmax) and (abs(new_x-old_x) > max(tol,tol*abs(new_x))):
        new_y = f(new_x)
        next_x = new_x + (new_x-old_x)/(new_y-old_y)*(y-new_y)
        old_x = new_x
        old_y = new_y
        new_x = next_x
        it += 1
    if it == itmax:
        print("!!! max number of iterations (%2i) reached"%(it))
        callseq = ''
        for i in range(len(inspect.stack())):
            callseq += '/'+inspect.stack()[i][3]
        print('    from',callseq)
    return new_x



"""
    The ``Isentropic`` module
    =========================
 
    Provides supersonic flow functions
 
    :Example:
 
    >>> import aerokit.aero.Supersonic as sup
    >>> Is.TiTs_Mach(1.)
    1.2
    >>> Is.TiTs_Mach(2., gamma=1.6)
    2.2
 
    Available functions
    -------------------
 
 """

import math
import numpy as np
from aerokit.aero import IterativeSolve
from aerokit.aero import Isentropic as Is
from aerokit.aero import degree     as deg
from aerokit.common import defaultgas as defg # relative import is deprecated by doctest
from scipy.optimize import newton

# -- 2D supersonic invariants --

def PrandtlMeyer_Mach(Mach, gamma=defg._gamma):
    """
        Computes Prandtl-Meyer (or Busemann) function from Mach number

        Long comment
 
        :param Mach:  local Mach number
        :param gamma: specific heat ratio, default from aerokit.common.defaultgas
        :return:      result omega (degree)
 
        :Example:

        >>> PrandtlMeyer_Mach(2.) # with default gamma 1.4
        26.379760813416446

        .. seealso:: 
        .. note:: available for scalar or array (numpy) computations
    """     
    g    = gamma
    gm1  = gamma - 1.
    beta = np.sqrt(Mach**2-1.)
    cg   = np.sqrt((g+1.)/gm1)
    return (cg*np.arctan(beta/cg) - np.arctan(beta))*(180./math.pi)

def old_Mach_PrandtlMeyer(omega, gamma=defg._gamma):
    def omega_of_mach(m):
        return PrandtlMeyer_Mach(m, gamma)
    return IterativeSolve.secant_solve(omega_of_mach, omega, 2.)

def Mach_PrandtlMeyer(omega, gamma=defg._gamma):
    def omega_of_mach(m):
        return PrandtlMeyer_Mach(m, gamma)-omega
    #return IterativeSolve.secant_solve(omega_of_mach, omega, 2.)
    result = newton(omega_of_mach, 2.+0.*omega)
    return result if np.size(result)!=1 else np.asscalar(result)

def Mach_PMFmmu(om_m_mun, gamma=defg._gamma):
    "return Mach number from omega-mu value (in degree)"
    g    = gamma
    gm1  = gamma - 1.
    cg   = np.sqrt((g+1.)/gm1)
    return np.sqrt((cg*deg.tan((om_m_mun + 90.)/cg))**2+1.)

def deflection_Mach_IsentropicPsratio(Mach, Pratio, gamma=defg._gamma):
    m2 = Is.Mach_PiPs(Is.PiPs_Mach(Mach, gamma)/Pratio, gamma)
    return -PrandtlMeyer_Mach(Mach, gamma)+PrandtlMeyer_Mach(m2, gamma)

def IsentropicPsratio_Mach_deflection(Mach, dev, gamma=defg._gamma):
    m2 = Mach_PrandtlMeyer(PrandtlMeyer_Mach(Mach, gamma)-dev, gamma)
    return Is.PiPs_Mach(Mach, gamma)/Is.PiPs_Mach(m2, gamma)



# ===============================================================
# automatic testing

if __name__ == "__main__":
    import doctest
    doctest.testmod()


# -*- coding: utf-8 -*-
"""
Description
-----------

Module containing the definition of a DicoVar object, which allow the switch
from different variable denomination in vtk output files, depending on which
solver denomination you use (CharlesX, OpenFOAM, Hybrid, ...)

Contains
--------
dicoVar : _DicoVar object
    The current database for variables denomination
setDicoVar(software) : function
    Set the current database to a given software denomination
printDatabase() : function
    Print the current database denomination

Examples
--------
When implementing a vtk file manipulation function

>>> # Import dicovar module database
>>> from aerokit.common.dicovar import dicoVar as dv
>>> # test if the field U_AVG is present
>>> if data_outVTK.GetPointData().HasArray(dv('u_avg')) != 1:
>>>     raise ValueError("Error : field U_AVG not present")
>>> # test if the field RHO_AVG is present
>>> if data_outVTK.GetPointData().HasArray(dv['rho_avg']) != 1:
>>>     raise ValueError("Error : field RHO_AVG not present")
>>> # test if the field TauWallAvg is present
>>> if wall_outVTK.GetPointData().HasArray(dv.get('tauw_avg')) != 1:
        raise ValueError("Error : field TauWallAvg not present")

When implementing a script associated to a given software

>>> # Import dicovar module functions
>>> from aerokit.common.dicovar import setDicoVar, printDicoVar
>>> # Setting CharlesX dicovar
>>> setDicoVar('cx')
>>> printDicoVar()

Notes
-----
- The implementation allow to use several names to set one software (ex : 'cx',
  'CharlesX', ... for CharlesX denomination)
- Several ways to get a variables : dv[...], dv(...), dv.get(...)
- variable name handles capital letters : dv['U_avg'] <=> dv['u_avg']
"""
import copy as _copy

# -----------------------------------------------------------------
# Part to modify in order to implement other variable denominations
# ---------------

# CharlesX variables dico
_cxNames = ['IC3', 'ic3', 'CharlesX', 'cx', 'charlesx']
_cxDico = {
    'P':   'P',
    'p':   'P',
    'Ps':  'P',
    'rho': 'RHO',
    'T':   'T',
    'V':  'U',
    'Vx': 'U_X',
    'Vy': 'U_Y',
    'Vz': 'U_Z',
    'u_avg': 'U_AVG',
    'rho_avg': 'RHO_AVG',
    'tauw_avg': 'TauWallAvg',
    'mulam_avg': 'MU_LAM_AVG',
    't_avg': 'T_AVG',
    'u_rms': 'U_RMS',
    'u_rey': 'U_REY'}

# OpenFOAM variables dico (just examples, to modify ...)
_ofNames = ['OpenFOAM', 'of', 'openfoam', 'ofoam']
_ofDico = {
    'u_avg': 'Uavg',
    'rho_avg': 'rhoAVG',
    'tauw_avg': '???',
    'mulam_avg': '???',
    't_avg': '???',
    'u_rms': '???',
    'u_rey': '???'}

# Database with all software denominations
_dataBase = [[_cxNames, _cxDico],
             [_ofNames, _ofDico]]

_default = 'IC3'


# -----------------------------------------------------------------
# DO NOT MODIFY THIS PART ...
# -----------------------

class _DicoVar(object):
    """
    Class representing a database for variables denomination of CFD
    softwares
    """

    def __init__(self, software):

        # Internal variables definitions
        self._dv = {}
        self._softName = ''

        # Setting database
        softOK = False
        for soft in _dataBase:
            if software.lower() in soft[0]:
                self._dv = _copy.deepcopy(soft[1])
                self._softName = soft[0][0]
                softOK = True
                print('---- HADES::DicoVar ----')
                print('--> Using {} variables denomination'.format(self._softName))
        if not softOK:
            raise ValueError('Not variable denomination database for {}'
                             .format(software))

    # Main class methods
    def keys(self):
        """
        return list of keys in dictionnary
        """
        return self._dv.keys()            

    def get(self, var):
        """
        Get the variable name with its denomination in the selected software database
        Parameters:
            var : String            The DaePy denomination of the variable
        Returns:
            varSoftware : string            The selected software denomination of the variable
        """
        if var in self._dv:
            return self._dv[var]
        else:
            raise ValueError('{} variable not in {} database dictionnary'.format(var, self._softName))

    def addVar(self, var, sVar):
        """
        Add a variable name with its denomination in the selected software database
        Parameters :
            var : String        The DaePy denomination of the variable (not in database)
            sVar : string       The software denomination of the variable
        """
        if var not in self._dv:
            self._dv[var] = sVar
        else:
            raise ValueError('{} variable already in {}'.format(var, self._softName) + 'database dictionnary.\n' +'Use setVar function instead')
        
    def setVar(self, var, sVar):
        """
        Set a variable name with a new denomination in the selected software database
        Parameters
        ----------
        var : String            The DaePy denomination of the variable (already in database)
        sVar : String           The new software denomination of the variable
        """
        if var in self._dv:
            self._dv[var] = sVar
            print('---- HADES::DicoVar ----')
            print('--> Changing denomination of {} into {}'.format(var, sVar))
        else:
            raise ValueError('{} variable not in {} database dictionnary'.format(var, self._softName))

    def printDatabase(self):
        """Print the variables denomination for the selected software"""
        print('---- HADES::DicoVar ----')
        print('--> Database dictionnary for {}'.format(self._softName))
        for elt in self._dv:
            print('      {} => {}'.format(elt, self._dv[elt]))

    # Definition of other ways to get a variable
    def __call__(self, var):
        return self.get(var)

    def __getitem__(self, var):
        return self.get(var)




# The main _DicoVar variable
dicoVar = _DicoVar(_default)


def setDicoVar(software):
    """
    Associate the denomination database to a given software

    Parameters
    ----------
    software : String
        The selected software (CharlesX, OpenFOAM, Hybrid, ...)
    """
    dicoVar.__init__(software)


def printDicoVar():
    """Print the variables denominations for the selected software"""
    dicoVar.printDatabase()

# -----------------------------------------------------------------




"""@package ShockWave
  locla Rankine Hugoniot equations for shock waves
"""

import math
import aerokit.aero.IterativeSolve as ITS
import numpy      as np
import aerokit.aero.Isentropic as Is
import aerokit.aero.degree as degree
from aerokit.common import defaultgas as defg # relative import is deprecated by doctest

# --- NORMAL SHOCK WAVE ---

def Ps_ratio(Mn, gamma=defg._gamma):
    return 1.+ (2.*gamma/(gamma+1.))*(Mn**2-1.)

def Mn_Ps_ratio(Pratio, gamma=defg._gamma):
    return np.sqrt(1+(Pratio-1.)*(gamma+1.)/(2.*gamma)) 

def Rho_ratio(Mn, gamma=defg._gamma):
    return ((gamma+1.)*Mn**2)/(2.+(gamma-1.)*Mn**2)

def Ts_ratio(Mn, gamma=defg._gamma):
    return Ps_ratio(Mn, gamma)/Rho_ratio(Mn, gamma)

def downstream_Mn(Mn, gamma=defg._gamma):
    return np.sqrt((1.+.5*(gamma-1.)*Mn**2)/(gamma*Mn**2-.5*(gamma-1.)))

def Pi_ratio(Mn, gamma=defg._gamma):
    return Ps_ratio(Mn, gamma)*Is.PiPs_Mach(downstream_Mn(Mn, gamma))/Is.PiPs_Mach(Mn, gamma)

def Mn_Pi_ratio(piratio, gamma=defg._gamma):
    def piratio_of_mach(m):
        return Pi_ratio(m, gamma)
    if piratio > 1:
        print("!!! cannot find Mn for Pi_ratio > 1")
        return 1
    return ITS.secant_solve(piratio_of_mach, piratio, 1.5)

# --- LOCAL 2D SHOCK WAVE ---

def deflection_Mach_sigma(Mach, sigma, gamma=defg._gamma):
    sigrad = np.radians(sigma)
    Mn     = Mach*np.sin(sigrad)
    return sigma-np.degrees(np.arctan2(np.tan(sigrad), Rho_ratio(Mn, gamma)))

def deflection_Mach_ShockPsratio(Mach, Pratio, gamma=defg._gamma):
    return deflection_Mach_sigma(Mach, degree.asin(Mn_Ps_ratio(Pratio, gamma)/Mach), gamma)

def downstreamMach_Mach_ShockPsratio(Mach, Pratio, gamma=defg._gamma):
    Mn0 = Mn_Ps_ratio(Pratio, gamma)
    sig = degree.asin(Mn0/Mach)
    dev = deflection_Mach_sigma(Mach, sig, gamma)
    Mn1 = downstream_Mn(Mn0, gamma)
    return Mn1/degree.sin(sig-dev)

def weaksigma_Mach_deflection(Mach, deflection, gamma=defg._gamma):
    ka = (1. + .5*(gamma+1.)*Mach**2)*degree.tan(deflection)
    kb = 1. - Mach**2
    kc = (1. + .5*(gamma-1.)*Mach**2)*degree.tan(deflection)
    kd = (ka**2/3. - kb)/3.
    ke = 2.*ka**3/27. - ka*kb/3. + kc
    if ke**2 - 4.*kd**3 > 0:
        print("no weak shock wave solution")
        return degree.asin(1./Mach)
    else:
        phi = np.arccos(-.5*ke/np.sqrt(kd**3))
        kf  = 2.*np.sqrt(kd)*np.cos(phi/3.) - ka/3.
        return degree.atan(1./kf)

def strongsigma_Mach_deflection(Mach, deflection, gamma=defg._gamma):
    ka = (1. + .5*(gamma+1.)*Mach**2)*degree.tan(deflection)
    kb = 1. - Mach**2
    kc = (1. + .5*(gamma-1.)*Mach**2)*degree.tan(deflection)
    kd = (ka**2/3. - kb)/3.
    ke = 2.*ka**3/27. - ka*kb/3. + kc
    if ke**2 - 4.*kd**3 > 0:
        print("no strong shock wave solution")
        return 90.
    else:
        phi = np.acos(-.5*ke/np.sqrt(kd**3)) + 4*math.pi
        kf  = 2.*np.sqrt(kd)*np.cos(phi/3.) - ka/3.
        return degree.atan(1./kf)

def sigma_Mach_deflection(Mach, deflection, gamma=defg._gamma):
    """
    computes the shock angle from upwstream Mach number and deflection angle
    """
    def local_f(sig):
        return deflection_Mach_sigma(Mach, sig, gamma)
    return ITS.secant_solve(local_f, np.abs(deflection), degree.asin(1./Mach)+deflection)*np.sign(deflection)

def dev_Max(Mach, gamma=defg._gamma):
    """ computes the maximum deviation (always subsonic downstream flow), separation of weak/strong shock
    """
    return deflection_Mach_sigma(Mach, sigma_DevMax(Mach, gamma=gamma), gamma=gamma)

def dev_Sonic(Mach, gamma=defg._gamma):
    """ computes the deviation angle for a downstream SONIC Mach number
    """
    return deflection_Mach_sigma(Mach, sigma_Sonic(Mach, gamma=gamma), gamma=gamma)

def sigma_DevMax(Mach, gamma=defg._gamma):
    """ computes the shock angle at maximum deviation (always subsonic downstream flow), separation of weak/strong shock
    """
    fogpu = 4./(gamma+1.)
    M2    = np.square(Mach)
    #ka = (M2-1.)*(1.+.5*(gamma-1.)*M2)
    #kb = .25*((gamma+1.)*M2-(3.-gamma))*M2 + 1.
    #return degree.atan(np.sqrt((kb+np.sqrt(kb**2+ka))/ka))
    return degree.asin(np.sqrt(1./fogpu/gamma/M2*(M2-fogpu+np.sqrt(M2*M2+2*(gamma-1.)*fogpu*M2+4.*fogpu))))

def sigma_Sonic(Mach, gamma=defg._gamma):
    """ computes the shock angle for a downstream SONIC Mach number
    """
    M2 = np.square(Mach)
    ka = gamma-3+M2*(gamma+1)
    kb = (gamma+1)*(np.square(M2-3)+gamma*np.square(M2+1))
    return degree.asin(np.sqrt((ka+np.sqrt(kb))/(4*gamma*M2)))

# --- CONICAL SHOCK WAVE ---

def conical_deflection_Mach_sigma(Mach, sigma, gamma=defg._gamma, tol=1.0e-6):
    def rkf45(F, x, y, h):
        # Runge-Kutta-Fehlberg formulas
        C = [37./378, 0., 250./621, 125./594, 0., 512./1771]
        D = [2825./27648, 0., 18575./48384, 13525./55296, 277./14336, 1./4]
        n = len(y)
        K = numpy.zeros((6,n))
        K[0] = h*F(x,y)
        K[1] = h*F(x + 1./5*h, y + 1./5*K[0])
        K[2] = h*F(x + 3./10*h, y + 3./40*K[0] + 9./40*K[1])
        K[3] = h*F(x + 3./5*h, y + 3./10*K[0]- 9./10*K[1] + 6./5*K[2])
        K[4] = h*F(x + h, y - 11./54*K[0] + 5./2*K[1] - 70./27*K[2] + 35./27*K[3])
        K[5] = h*F(x + 7./8*h, y + 1631./55296*K[0] + 175./512*K[1] + 575./13824*K[2] + 44275./110592*K[3] + 253./4096*K[4])
        # Initialize arrays {dy} and {E}
        E  = numpy.zeros((n))
        dy = numpy.zeros((n))
        # Compute solution increment {dy} and per-step error {E}
        for i in range(6):
            dy = dy + C[i]*K[i]
            E  = E + (C[i] - D[i])*K[i]
        # Compute RMS error e
        e = np.sqrt(sum(E**2)/n)
        return dy, e

    def rhs(phi, data):
        th = data[0]
        ma = data[1]
        k  = 1.-(ma*degree.sin(phi-th))**2
        rhs=numpy.zeros(2)
        rhs[0] = -degree.sin(th)*degree.cos(phi-th)/degree.sin(phi)/k
        rhs[1] =  np.pi/180.*degree.sin(th)*degree.sin(phi-th)/degree.sin(phi)/k*ma*(1.+.5*(gamma-1)*ma*ma)
        return rhs

    th   = deflection_Mach_sigma(Mach, sigma, gamma)
    ma   = downstream_Mn(Mach*degree.sin(sigma), gamma)/degree.sin(sigma-th)
    phi  = sigma
    thma = numpy.array([th, ma])
    h    = -phi/10.
    conv = phi
    while (abs(conv) >= tol):
        dthma, err = rkf45(rhs, phi, thma, h)
        # Accept integration step if error e is within tolerance
        if err <= tol:
            conv = thma[0] - phi
            if conv/(h-dthma[0]) < 1:
                h = h*conv/(h-dthma[0])
            else:
                thma = thma + dthma
                phi  = phi  + h
                print(phi, thma)
        else:
            h = 0.9*h*(tol/err)**0.2
            print("new h ",h)

    deflection = .5*(thma[0] + phi)
    return deflection

def conical_sigma_Mach_walldeflection(Mach, deflection, gamma=defg._gamma):
    def local_f(sig):
        return conical_deflection_Mach_sigma(Mach, sig, gamma)
    return ITS.secant_solve(local_f, deflection, degree.asin(1./Mach)+deflection)




"""@package model1D
  representation of 1D flows
"""

import math
import numpy     as np
#import ShockWave as sw
#import IterativeSolve
from ..common   import defaultgas as defg
from aerokit.aero import Isentropic as Is

# -- class --

class state():
	"""
	defines a one dimensional state class

	Attributes:
		_gamma
		rho
		u
		p
	"""
	def __init__(self, rho, u, p, gamma=defg._gamma):
	 	self._gamma = gamma
	 	self.rho    = rho
	 	self.u      = u
	 	self.p      = p

	def __repr__(self):
		return "state (rho, u, p) : (%s, %s, %s)" % (self.rho, self.u, self.p)

	def copy(self):
		return state(self.rho, self.u, self.p, self._gamma)

	def compute_from_pt_rtt_M(self, pt, rtt, M):
		ps  = pt /Is.PiPs_Mach(M, self._gamma)
		rts = rtt/Is.TiTs_Mach(M, self._gamma)
		self.__init__(rho=ps/rts, u=M*np.sqrt(self._gamma*rts), p=ps)
		
	def compute_from_pt_rtt_u(self, pt, rtt, u):
		gam   = self._gamma
		gsgmu = gam/(gam-1.)
		a2    = gam * rtt - .5*(gam-1.)*u**2
		ps    = pt / ( 1. + .5*u**2 / (gsgmu*rtt - .5*u**2) )**gsgmu
		self.__init__(rho=gam * ps / a2, u=u, p=ps)

	def compute_from_pt_rtt_p(self, pt, rtt, p):
		M   = Is.Mach_PiPs(pt/p, self._gamma)
		rts = rtt/Is.TiTs_Mach(M, self._gamma)
		self.__init__(rho=p/rts, u=M*np.sqrt(self._gamma*rts), p=p)
		
	def asound(self):
		"""returns speed of sound"""
		return math.sqrt(self._gamma*self.p/self.rho)

	def left_acoustic(self):
		"""returns left 'actual' speed of sound"""
		return self.u-math.sqrt(self._gamma*self.p/self.rho)

	def right_acoustic(self):
		"""returns left 'actual' speed of sound"""
		return self.u+math.sqrt(self._gamma*self.p/self.rho)

	def Mach(self):
		"""returns Mach number"""
		return self.u/self.asound()

	def massflow(self):
		"""returns massflow"""
		return self.rho * self.u

	def Ptot(self):
		"""returns Total pressure"""
		return self.p*Is.PiPs_Mach(self.Mach(), self._gamma)

	def rTtot(self):
		"""returns Total temperature"""
		return self.p/self.rho + .5*(self._gamma-1.)/self._gamma*self.u**2

	# def rTtot(self):
	# 	"""returns speed of sound"""
	# 	return self.u/self.asound()




""" 
	The ``unsteady1D`` module
	=========================
 
	Specific functions to compute propagation and evolution of 1D state
 
	:Example:
 
    >>> import aerokit.aero.Isentropic as Is
    >>> Is.TiTs_Mach(1.)
    1.2
    >>> Is.TiTs_Mach(2., gamma=1.6)
    2.2
 
    Available functions
    -------------------
 
    Blabla
  
"""
import math
import copy
import numpy     as np
import aerokit.aero.ShockWave as sw
import aerokit.aero.model1D   as model1D

# -- class --

class unsteady_state(model1D.state):
	"""
	defines a one dimensional state
	"""
	def copy(self):
		return copy.deepcopy(self)

	def copysymmetric(self):
		return unsteady_state(self.rho, -self.u, self.p, self._gamma)

	def _rankinehugoniot_from_ushock(self, ushock):
		"""
			returns Rankine-Hugoniot state, given a shock velocity
		
			..warning:: this function is made private because there is no test about upstream/downstream consistency
			nor the right ushock range (greater than u+a or lesser than u-a
		"""
		loc_Mn    = (self.u-ushock)/self.asound()          # this Mach number is signed
		rho_ratio = sw.Rho_ratio(loc_Mn, self._gamma)
		return unsteady_state(self.rho * rho_ratio,
		             ushock + (self.u-ushock)/rho_ratio,
		             self.p * sw.Ps_ratio(loc_Mn, self._gamma),
	 	             gamma=self._gamma)

	def delta_u_expansion(self, p):
		"""
			computes delta u through expansion given 'self' initial state and final pressure

			usage
			-----

			if C- expansion, from left state,  middle velocity ustar is (uL - delta_u_expansion)
			is C+ expansion, from right state, middle velocity ustar is (uR + delta_u_expansion)

			..warning:: There is no consistent test that (arg) p is lesser than self.p
			..note:: from Toro, page 119
		"""
		gmusd = .5*(self._gamma-1.)
		return self.asound()/gmusd*((p/self.p)**(gmusd/self._gamma)-1.)

	def delta_u_shock(self, p):
		"""
			computes delta u through shock given 'self' initial state and final pressure

			usage
			-----

			if C- shock, from left state,  middle velocity ustar is (uL - delta_u_shock)
			is C+ shock, from right state, middle velocity ustar is (uR + delta_u_shock)

			..warning:: There is no consistent test that (arg) p is greater than self.p
		"""
		return (p-self.p)*math.sqrt(2./self.rho/((self._gamma+1.)*p + (self._gamma-1.)*self.p))

	def rho_through_shock(self, p):
		"""
			computes rho increase through shock given 'self' initial state and final pressure

			..warning:: There is no consistent test that (arg) p is greater than self.p
		"""
		gmusgpu = (self._gamma-1.)/(self._gamma+1.)
		pratio  = p/self.p
		return self.rho * (pratio+gmusgpu)/(1.+gmusgpu*pratio)

	def rho_through_isentropic(self, p):
		"""
			computes rho increase through isentropic compression or expansion given 'self' initial state and final pressure
		"""
		return self.rho * (p/self.p) ** (1./self._gamma)

	def shock_downstream(self, ushock):
		"""computes downstream state of genuine unsteady shock wave"""

		### TO BE IMPLEMENTED# -- functions --




import numpy as np

def cos(a):
    return np.cos(np.radians(a))

def sin(a):
    return np.sin(np.radians(a))

def tan(a):
    return np.tan(np.radians(a))

def acos(a):
    return np.degrees(np.arccos(a))

def asin(a):
    return np.degrees(np.arcsin(a))

def atan(a):
    return np.degrees(np.arctan(a))

def atan2(a, b):
    return np.degrees(np.arctan2(a, b))


"""@package Propulsion
  thrust flow function
"""

import Isentropic
import MassFlow

def ThrustFunction(Mach, gamma=1.4):
    return MassFlow.Sigma_Mach(Mach, gamma)/Isentropic.PiPs_Mach(Mach, gamma)*(1.+gamma*Mach**2)

def Mach_ThrustFunction(thrust, Mach=2., gamma=1.4):
    def thrust_of_mach(m):
        return ThrustFunction(m, gamma)
    return IterativeSolve.secant_solve(thrust_of_mach, thrust, Mach)




""" 
	The ``inflow`` module
	=========================
 
	Provides class and function for inflow solvers for 1D unsteady flows
 
	:Example:
 
    >>> import aerokit.aero.Isentropic as Is
    >>> Is.TiTs_Mach(1.)
    1.2
    >>> Is.TiTs_Mach(2., gamma=1.6)
    2.2
 
    Available functions
    -------------------
 
    Blabla
  
"""

import math
import numpy     as np
from scipy.optimize import fsolve
import aerokit.aero.ShockWave  as sw
import aerokit.aero.unsteady1D as uq

# ===============================================================
# implementation of inflow_PB class

class inflow_pb():
	"""
		Class inflow_pb for unsteady 1D Euler

		attributes:
			_qinit  : initial state before flow variation, is qR state for equivalent Riemann problem
			_pstar  : static pressure at inflow
			_ustar  : velocity at inflow
			_qstarL : q state on time axis
			_qstarR : q state between q*L and qR
	"""
	def __init__(self, qinit):
	 	self._qR    = qinit.copy()

	def __repr__(self):
		return "(rho, u, p)*_L : (%s, %s, %s)\n" % (self._qstarL.rho, self._qstarL.u, self._qstarL.p) + \
			   "(rho, u, p)*_R : (%s, %s, %s)\n" % (self._qstarR.rho, self._qstarR.u, self._qstarR.p) + \
			   "(rho, u, p)_R  : (%s, %s, %s)"   % (self._qR.rho, self._qR.u, self._qR.p)

	def left_fastest(self):
		return self._waves[0]

	def right_fastest(self):
		return self._waves[4]

	def ustar(self):
		return self._ustar

	def pstar(self):
		return self._pstar

	def qstarL(self):
		return self._qstarL

	def qstarR(self):
		return self._qstarR

	def _delta_uR(self, p):
		"""right contribution to delta u in inflow problem"""
		if (p > self._qR.p):
			_delta = self._qR.delta_u_shock(p)
		else:
			_delta = self._qR.delta_u_expansion(p)
		return _delta

	def _pt0_from_pstar_rtt0(self, p, rtt0):
		ustar = self._qR.u + self._delta_uR(p)
		gam   = self._qR._gamma
		gsgmu = gam/(gam-1.)
		#print 'it',p, ustar, p * ( 1. + .5*ustar**2 / (gsgmu*rtt0 - .5*ustar**2) )**gsgmu
		return p * ( 1. + .5*ustar**2 / (gsgmu*rtt0 - .5*ustar**2) )**gsgmu

	def _pstar_estimate_expansion(self, pt0):
		"""exact solution for 2 expansion waves (for uniform gamma only)

			..note: from Toro, page 128
		"""
		return np.sqrt(self._qR.p * pt0)

	def solve_with_pt0_rtt0(self, pt0, rtt0):
		def dpt(p):
			return self._pt0_from_pstar_rtt0(p, rtt0) - pt0
		self._pstar  = fsolve(dpt, self._pstar_estimate_expansion(pt0), xtol=1e-10)
		self._ustar  = self._qR.u + self._delta_uR(self._pstar)
		# finalize qstarR state
		if (self._pstar > self._qR.p):
			_rho = self._qR.rho_through_shock(self._pstar)
			self._qstarR = uq.unsteady_state(rho=_rho, u=self._ustar, p=self._pstar, gamma=self._qR._gamma)
		else:
			_rho = self._qR.rho_through_isentropic(self._pstar)
			self._qstarR = uq.unsteady_state(rho=_rho, u=self._ustar, p=self._pstar, gamma=self._qR._gamma)
		#
		self._qstarL = self._qR.copy()
		self._qstarL.compute_from_pt_rtt_u(pt0, rtt0, self._ustar)
		return



"""@package shockpolar
  Plot shock polar curves
"""

import numpy as np
import aerokit.common.defaultgas as defg # relative import is deprecated by doctest
import aerokit.aero.degree      as deg
import aerokit.aero.Supersonic  as sup

import matplotlib.pyplot as plt
from aerokit.aero.plot.defaultstyle import *


def plot_theta_pressure(mach, dev_range, gamma=defg._gamma, npts=100, thet_init=0., p_init=1., curve='C+', devmax=False, sonic=False, color='k', linestyle='-', **kwargs):
    """
    	Plot isentropic polar curve in deviation / pressure ratio axes

		Long comment
 
		:param mach:       upstream Mach number
        :param gamma:      specific heat ratio, default from aerokit.common.defaultgas
        :param npts:       number of computed points, curve accuracy
        :param thet_init:  upstream angle (shift the curve by this angle), default 0.
        :param p_init:     reference pressure (shift the curve by this ratio), default 1.
        :param curve:      choose which curve to plot (left, right or both)
		:return:     
 
 		:Example:

		.. seealso:: 
		.. note:: 
    """	
 
    dev = np.linspace(dev_range[0], dev_range[1], npts+1)
    ps  = p_init * sup.IsentropicPsratio_Mach_deflection(mach, dev, gamma=gamma)     # pressure ratio only depends on normal Mach number
    if curve in ['c+', 'C+', 'both']: plt.plot(thet_init+dev, ps, color=color, linestyle=linestyle, **kwargs)
    ps  = p_init * sup.IsentropicPsratio_Mach_deflection(mach, -dev, gamma=gamma)     # pressure ratio only depends on normal Mach number
    if curve in ['c-', 'C-', 'both']: plt.plot(thet_init+dev, ps, color=color, linestyle=linestyle, **kwargs)



"""@package  CompressibleFlow
  legacy package importing legacy functions
"""

from . import Isentropic as ist
from . import Supersonic as sup
from . import MassFlow   as mfl

# -- isentropic functions --

def TiTs_Mach(Mach, gamma=1.4):
    """ legacy interface to same function in Isentropic
    """
    return ist.TiTs_Mach(Mach, gamma)

def PiPs_Mach(Mach, gamma=1.4):
    """ legacy interface to same function in Isentropic
    """
    return ist.PiPs_Mach(Mach, gamma)

def Mach_TiTs(TiTs, gamma=1.4):
    """ legacy interface to same function in Isentropic
    """
    return ist.Mach_TiTs(TiTs, gamma)

def Mach_PiPs(PiPs, gamma=1.4):
    """ legacy interface to same function in Isentropic
    """
    return ist.Mach_PiPs(PiPs, gamma)

def Velocity_MachTi(Mach, Ti, r=287.1, gamma=1.4):
    """ legacy interface to same function in Isentropic
    """
    return ist.Velocity_MachTi(Mach, Ti, r, gamma)

# -- 2D supersonic invariants --

def PrandtlMeyer_Mach(Mach, gamma=1.4):
    """ legacy interface to same function in Supersonic
    """
    return sup.PrandtlMeyer_Mach(Mach, gamma)

def Mach_PrandtlMeyer(omega, gamma=1.4):
    """ legacy interface to same function in Supersonic
    """
    return sup.Mach_PrandtlMeyer(omega, gamma)

def deflection_Mach_IsentropicPsratio(Mach, Pratio, gamma=1.4):
    """ legacy interface to same function in Supersonic
    """
    return sup.deflection_Mach_IsentropicPsratio(Mach, Pratio, gamma)

def IsentropicPsratio_Mach_deflection(Mach, dev, gamma=1.4):
    """ legacy interface to same function in Supersonic
    """
    return sup.IsentropicPsratio_Mach_deflection(Mach, dev, gamma)

# -- Compressible flow functions  --

def WeightMassFlow(Mach, r=287.1, gamma=1.4):
    """ legacy interface to same function in MassFlow
    """
    return mfl.WeightMassFlow(Mach, r, gamma)

def Sigma_Mach(Mach, gamma=1.4):
    """ legacy interface to same function in MassFlow
    """
    return mfl.Sigma_Mach(Mach, gamma)

def Mach_Sigma(sigma, Mach=2., gamma=1.4):
    """ legacy interface to same function in MassFlow
    """
    return mfl.Mach_Sigma(sigma, Mach, gamma)




# from Numerical Methods in Engineering With Python

from numpy import array,sum,zeros,float64
from math import sqrt

def RK4(F,x,y,xStop,h):
    def run_kut4(F,x,y,h):
        # Computes increment of y from Eqs. (7.10)
        K0 = h*F(x,y)
        K1 = h*F(x + h/2.0, y + K0/2.0)
        K2 = h*F(x + h/2.0, y + K1/2.0)
        K3 = h*F(x + h, y + K2)
        return (K0 + 2.0*K1 + 2.0*K2 + K3)/6.0
    X = []
    Y = []
    X.append(x)
    Y.append(y)
    while x < xStop:
        h = min(h,xStop - x)
        y = y + run_kut4(F,x,y,h)
        # Runge-Kutta Methods
        x = x + h
        X.append(x)
        Y.append(y)
    return array(X),array(Y)


def RKF45(F, x, y, xStop, h, tol=1.0e-6):
    def run_kut5(F, x, y, h):
        # Runge-Kutta-Fehlberg formulas
        C = [37./378, 0., 250./621, 125./594, 0., 512./1771]
        D = [2825./27648, 0., 18575./48384, 13525./55296, 277./14336, 1./4]
        n = len(y)
        K = numpy.zeros((6,n),type=float64)
        K[0] = h*F(x,y)
        K[1] = h*F(x + 1./5*h, y + 1./5*K[0])
        K[2] = h*F(x + 3./10*h, y + 3./40*K[0] + 9./40*K[1])
        K[3] = h*F(x + 3./5*h, y + 3./10*K[0]- 9./10*K[1] + 6./5*K[2])
        K[4] = h*F(x + h, y - 11./54*K[0] + 5./2*K[1] - 70./27*K[2] + 35./27*K[3])
        K[5] = h*F(x + 7./8*h, y + 1631./55296*K[0] + 175./512*K[1] + 575./13824*K[2] + 44275./110592*K[3] + 253./4096*K[4])
        # Initialize arrays {dy} and {E}
        E  = numpy.zeros((n),type=float64)
        dy = numpy.zeros((n),type=float64)
        # Compute solution increment {dy} and per-step error {E}
        for i in range(6):
            dy = dy + C[i]*K[i]
            E  = E + (C[i] - D[i])*K[i]
        # Compute RMS error e
        e = sqrt(sum(E**2)/n)
        return dy,e

    X = []
    Y = []
    X.append(x)
    Y.append(y)
    stopper = 0 # Integration stopper(0 = off, 1 = on)
    for i in range(10000):
        dy,e = run_kut5(F,x,y,h)
        # Accept integration step if error e is within tolerance
        if e <= tol:
            y = y + dy
            x = x + h
            X.append(x)
            Y.append(y)
            # Stop if end of integration range is reached
            if stopper == 1: break
        # Compute next step size from Eq. (7.24)
        if e != 0.0:
            hNext = 0.9*h*(tol/e)**0.2
        else: hNext = h
        # Check if next step is the last one; is so, adjust h
        if (h > 0.0) == ((x + hNext) >= xStop):
            hNext = xStop - x
            stopper = 1
        h = hNext
    return array(X),array(Y)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#

import matplotlib.pyplot as plt
import numpy as np
import aerokit.aero.Isentropic as Is

# Grandeurs d'entrée
P0 = 227.00e2
T0 = 217.00
M0 = 0.8

# Paramètres
xi_e = 0.97
eta_c = 0.90
eta_f = 0.89
eta_comb = 0.99
xi_cc = 0.95
eta_m = 0.98
eta_turb = 0.88
xi_tuy = 0.97

P_k = 42800.00e3

# Gaz parfait
r = 287.00
r_ = 291.60
gamma = 1.40
gamma_ = 1.33

def calculs(lambda_, pi_c, Tt4, m, taux_meca):
    V0 = M0 * np.sqrt(gamma * r * T0)
    # Calculs grandeurs totales entrée
    Pt0 = P0 * (1 + ((gamma - 1) / 2) * M0**2)**(gamma/(gamma - 1))
    Tt0 = T0 * (1 + ((gamma - 1) / 2) * M0**2)
    # Calculs plan 2
    Tt2 = Tt0
    Pt2 = Pt0*xi_e
    # Gaz parfait
    cp = (r * gamma)/(gamma - 1)
    cp_ = (r_ * gamma_)/(gamma_ - 1)
    # Calculs CHP
    m_c = m / (lambda_ + 1)
    Tt3 = Tt2 * pi_c**((gamma - 1)/(gamma * eta_c))
    Pt3 = Pt2 * pi_c
    w_uc = m_c * cp * (Tt3 - Tt2)
    # Calculs chambre de combustion
    m_k = m_c * (((cp_ * Tt4) - (cp * Tt3))/((eta_comb * P_k) - (cp_ * Tt4)))
    alpha = m_k / m_c
    Pt4 = Pt3 * xi_cc
    # Calculs turbine compresseur
    Tt4_ = Tt4 - ((w_uc)/(m_c * (1 + alpha) * cp_ * eta_m))
    pi_tc = (Tt4 / Tt4_)**(gamma_ / (eta_turb * (gamma_ - 1)))
    Pt4_ = Pt4 / pi_tc
    # Calculs turbine fan
    P5 = P0
    T5 = Tt4_ * (P5 / Pt4_)**((gamma_ - 1) / gamma_)
    w_uf_theo = m_c * (1 + alpha) * cp_ * (T5 - Tt4_)
    Wu_f = w_uf_theo * taux_meca
    Tt5 = (Wu_f / (cp_ * m_c * (1 + alpha))) + Tt4_
    pi_tf = (Tt4_ / Tt5)**(gamma_ / (eta_turb * (gamma_ - 1)))
    Pt5 = Pt4_ / pi_tf
    # Calculs tuyère corps
    Tt9 = Tt5
    Pt9 = Pt5 * xi_tuy
    P9 = P0
    M9 = np.sqrt(((Pt9/P9)**((gamma_ - 1)/gamma_ ) - 1) * (2/(gamma_ - 1)))
    T9 = Tt9 / (1 + ((gamma_ - 1)/2)*M9**2)
    A9 = np.sqrt(gamma_ * r_ * T9)
    V9 = M9 * A9
    Fc = m_c * (V9 - V0)
    # Calculs fan
    m_f = (lambda_ * m) / (lambda_ + 1)
    Wf = -w_uf_theo * taux_meca * eta_m
    Tt13 = Wf/(m_f * cp) + Tt2
    pi_f = (Tt13 / Tt2)**(eta_f * gamma / (gamma - 1))
    Pt13 = Pt2 * pi_f
    # Calculs tuyère fan
    Pt19 = Pt13 * xi_tuy
    Tt19 = Tt13
    P19 = P0
    M19 = np.sqrt(((Pt19/P19)**((gamma - 1)/gamma ) - 1) * (2/(gamma - 1)))
    T19 = Tt19 / (1 + ((gamma - 1)/2)*M19**2)
    A19 = np.sqrt(gamma * r * T19)
    V19 = M19 * A19
    print("V9  inl", V9, M9)
    print("V19 inl", V19)
    Ff = m_f * (V19 - V0)
    # Calculs performances
    F = Fc + Ff
    Fspe = F/m
    mk_spe = (m_k / F)*10e6
    Qf = P_k * m_k
    delta_E = .5*(m_c*((1 + alpha)*V9**2 - V0**2) + m_f*(V19**2 - V0**2))
    eta_th = delta_E / Qf
    eta_prop = (F * V0) / delta_E
    eta = eta_th * eta_prop
    return Fspe, mk_spe, eta_th, eta_prop, eta, pi_f
    
def plot(X,Y,num_figure,title,x_label,y_label):
    figure = plt.figure(num=num_figure, figsize=(12,7))
    figure.suptitle(title)
    sub1 = plt.subplot(len(Y), 1, 1)
    plt.ylabel(y_label[0])
    plt.plot(X,Y[0])
    for i in range(1,len(Y)) :
        plt.subplot(len(Y),1,i+1, sharex=sub1)
        plt.plot(X,Y[i])
        plt.ylabel(y_label[i])
    plt.xlabel(x_label)
    return figure

def main(lambda_min,lambda_max,pi_c_min,pi_c_max,Tt4_min,Tt4_max):
    nb_valeurs = 20
    lambda_base = 6.1
    pi_c_base = 32.8
    Tt4_base = 1410
    m = 180
    taux_meca = 0.5
    y_label = ["Fspe","mk_spe","eta_th","eta_prop","eta_global"]
    lambda_array = np.linspace(lambda_min,lambda_max, num=nb_valeurs)
    pi_c_array = np.linspace(pi_c_min,pi_c_max, num=nb_valeurs)
    Tt4_array = np.linspace(Tt4_min,Tt4_max, num=nb_valeurs)
    taux_meca_array = np.linspace(0.2,0.8, num=nb_valeurs)
    results = [[0]*nb_valeurs for i in range(5)]
    for i in range(0,lambda_array.size) :
        result = calculs(lambda_array[i],pi_c_base,Tt4_base,m,taux_meca)
        results[0][i] = result[0]
        results[1][i] = result[1]
        results[2][i] = result[2]
        results[3][i] = result[3]
        results[4][i] = result[4]
    figure1 = plot(lambda_array,results,1,"Performances 1","Taux de dilution",y_label)
    for i in range(0,pi_c_array.size) :
        result = calculs(lambda_base,pi_c_array[i],Tt4_base,m,taux_meca)
        results[0][i] = result[0]
        results[1][i] = result[1]
        results[2][i] = result[2]
        results[3][i] = result[3]
        results[4][i] = result[4]
    figure2 = plot(pi_c_array,results,2,"Performances 2","Taux de compression coeur",y_label)
    for i in range(0,Tt4_array.size) :
        result = calculs(lambda_base,pi_c_base,Tt4_array[i],m,taux_meca)
        results[0][i] = result[0]
        results[1][i] = result[1]
        results[2][i] = result[2]
        results[3][i] = result[3]
        results[4][i] = result[4]
    figure3 = plot(Tt4_array,results,3,"Performances 3","Temperature sortie de chambre de combustion",y_label)
##    for i in range(0,taux_meca_array.size) :
##        result = calculs(lambda_base,pi_c_base,Tt4_base,m,taux_meca_array[i])
##        results[0][i] = result[0]
##        results[1][i] = result[1]
##        results[2][i] = result[2]
##        results[3][i] = result[3]
##        results[4][i] = result[4]
##    figure4 = plot(taux_meca_array,results,4,"Performances 4","Taux fan",y_label)
    figure1.show()
    figure2.show()
    figure3.show()
#    figure4.show()


def calculs_tau_optimum(lambda_, pi_c, Tt4, m):
    V0 = M0 * (gamma * r * T0)**(1/2)
    # Calculs grandeurs totales entrée
    Pt0 = P0 * (1 + ((gamma - 1) / 2) * M0**2)**(gamma/(gamma - 1))
    Tt0 = T0 * (1 + ((gamma - 1) / 2) * M0**2)
    # Calculs plan 2
    Tt2 = Tt0
    Pt2 = Pt0*xi_e
    # Gaz parfait
    cp = (r * gamma)/(gamma - 1)
    cp_ = (r_ * gamma_)/(gamma_ - 1)
    # Calculs fan
    tau_lambda = (cp_ * Tt4)/(cp * Tt0)
    tau_r = Tt0/T0
    tau_c = pi_c**((gamma - 1)/(eta_c * gamma))
    tau_f = (tau_lambda*tau_r*(tau_c - 1) - (tau_lambda/(tau_r*tau_c)) + lambda_*tau_r + 1)/(tau_r*(1 + lambda_))
    pi_f = tau_f**((eta_f*gamma)/(gamma - 1))
    Tt13 = Tt2 * tau_f
    Pt13 = Pt2 * pi_f
    m_f = (lambda_ * m) / (lambda_ + 1)
    wu_f = m_f * cp * (Tt13 - Tt2)
    # Calculs CHP
    m_c = m / (lambda_ + 1)
    Tt3 = Tt2 * pi_c**((gamma - 1)/(gamma * eta_c))
    Pt3 = Pt2 * pi_c
    wu_c = m_c * cp * (Tt3 - Tt2)
    # Calculs chambre de combustion
    m_k = m_c * (((cp_ * Tt4) - (cp * Tt3))/((eta_comb * P_k) - (cp_ * Tt4)))
    alpha = m_k / m_c
    Pt4 = Pt3 * xi_cc
    # Calculs turbine compresseur
    Tt5 = Tt4 - ((wu_c + wu_f)/(m_c * (1 + alpha) * cp_ * eta_m))
    pi_tc = (Tt4 / Tt5)**(gamma_ / (eta_turb * (gamma_ - 1)))
    Pt5 = Pt4 / pi_tc
    # Calculs tuyère corps
    Tt9 = Tt5
    Pt9 = Pt5 * xi_tuy
    P9 = P0
    M9 = (((Pt9/P9)**((gamma_ - 1)/gamma_ ) - 1) * (2/(gamma_ - 1)))**(1/2)
    T9 = Tt9 / (1 + ((gamma_ - 1)/2)*M9**2)
    A9 = (gamma_ * r_ * T9)**(1/2)
    V9 = M9 * A9
    Fc = m_c * (V9 - V0)
    # Calculs tuyère fan
    Pt19 = Pt13 * xi_tuy
    Tt19 = Tt13
    P19 = P0
    M19 = (((Pt19/P19)**((gamma - 1)/gamma ) - 1) * (2/(gamma - 1)))**(1/2)
    T19 = Tt19 / (1 + ((gamma - 1)/2)*M19**2)
    A19 = (gamma * r * T19)**(1/2)
    V19 = M19 * A19
    Ff = m_f * (V19 - V0)
    # Calculs performances
    F = Fc + Ff
    Fspe = F/m
    mk_spe = (m_k / F)*10e6
    Qf = P_k * m_k
    delta_E = (1/2)*(m_c*((1 + alpha)*V9**2 - V0**2) + m_f*(V19**2 - V0**2))
    eta_th = delta_E / Qf
    eta_prop = (F * V0) / delta_E
    eta = eta_th * eta_prop
    print(tau_f)
    print(tau_r)
    print(tau_c)
    print(tau_lambda)
    print(Tt13)
    #return [Fspe, mk_spe, eta_th, eta_prop, eta]






"""
    The ``gasgenerator`` module
    =========================
 
    Provides standart Stagnation to Static Ratios for Ideal gas
 
    :Example:
 
    >>> import aerokit.aero.Isentropic as Is
    >>> Is.TiTs_Mach(1.)
    1.2
    >>> Is.TiTs_Mach(2., gamma=1.6)
    2.2
 
    Available functions
    -------------------
 
    Provides Ti/Ts Pi/Ps ratios from Mach number and reversed functions.
    Specific heat ratio `gamma` is optionnal and can be specified in the functions itself
    or using aerokit.common.defaultgas module
 """

import math
import numpy as np
from aerokit.common import defaultgas as defg # relative import is deprecated by doctest
import aerokit.aero.Isentropic as Is

# ===============================================================
# implemented functions

class base():
    """
    	virtual class for gas generator and other engine module

		Long comment
 
		:param Mach:  local Mach number
        :param gamma: specific heat ratio, default from aerokit.common.defaultgas
		:return:      result Ti/Ts ratio
 
 		:Example:

		>>> TiTs_Mach(1.) # with default gamma 1.4
		1.2

		.. seealso:: 
		.. note:: available for scalar or array (numpy) computations
    """	
    def __init__(self, OPR, Tt4, gam_cold=defg._gamma, gam_hot=defg._gamma, r_cold=defg._r, r_hot=defg._r,
                                 xi_inlet=1., xi_cc=1., eta_cc=1., Pci=42.8e6,
                                 etapolCHP=1., etapolTHP=1., eta_shaft=1.):
        self.OPR = OPR
        self.Tt4 = Tt4
        self.Pci = Pci
        self.gam_cold  = gam_cold
        self.gam_hot   = gam_hot
        self.r_cold    = r_cold
        self.r_hot     = r_hot
        self.xi_cc     = xi_cc
        self.xi_inlet  = xi_inlet
        self.etapolCHP = etapolCHP
        self.etapolTHP = etapolTHP
        self.eta_shaft = eta_shaft

    def flight_conditions(self, T0=1., P0=1., M0=0.):
        self.T0 = T0
        self.P0 = P0
        self.M0 = M0

    def update(self):
        gc  = self.gam_cold
        cpc = gc*self.r_cold/(gc-1.)
        self.Tt0 = self.T0*Is.TiTs_Mach(self.M0, gamma=self.gam_cold)
        self.Pt0 = self.P0*Is.PiPs_Mach(self.M0, gamma=self.gam_cold)
        self.V0  = self.M0*np.sqrt(gc*self.r_cold*self.T0)
        self.Pt2 = self.Pt0*self.xi_inlet
        self.Tt2 = self.Tt0
        self.Pt3 = self.Pt2*self.OPR
        self.Tt3 = self.Tt2*self.OPR**((gc-1.)/(gc*self.etapolCHP))
        #self.Tt4 = Ti_4
        self.Pt4 = self.Pt3*self.xi_cc
        gh  = self.gam_hot
        cph = gh*self.r_hot/(gh-1.)
        self.far = (cph*self.Tt4 - cpc*self.Tt3)/(self.xi_cc*self.Pci - cph*self.Tt4)
        self.Tt45 = self.Tt4 - cpc*(self.Tt3-self.Tt2)/(self.eta_shaft*cph*(1.+self.far))
        #print self.Tt3, self.Pt4, self.Tt45, self.Tt4, self.far
        self.Pt45 = self.Pt4*(self.Tt45/self.Tt4)**(gh/((gh-1.)*self.etapolTHP))

    def Wsp_cc(self):
        return self.far * self.Pci

    def Wsp_TBP_ideal(self, P19=None):
        if P19==None: P19=self.P0
        gogm1 = self.gam_hot/(self.gam_hot-1.)
        return (1.+self.far)*gogm1*self.r_hot/(self.gam_hot-1.)*self.Tt45*((P19/self.Pt45)**(1./gogm1)-1.)

    def thermal_efficiency(self):
        return self.Wsp_TBP_ideal()/self.Wsp_cc()

# ===============================================================
# automatic testing

if __name__ == "__main__":
    import doctest
    doctest.testmod()


"""@package MassFlow
  Massflow for compressible flow
"""

import math
import numpy as np
#from . import IterativeSolve
from scipy.optimize import fsolve, newton
from ..common import defaultgas as defg

# internal initialization Mach for sigma computations
def __MachSub_sigma(sigma, gamma):
	return (2./(gamma+1.))**(.5*(gamma+1.)/(gamma-1.))/sigma

def __MachSup_sigma(sigma, gamma):
	return np.sqrt(1.+1./sigma)

# -- Compressible flow functions  --

def WeightMassFlow(Mach, r=287.1, gamma=defg._gamma):
    return np.sqrt(gamma)*Mach*(1.+.5*(gamma-1)*Mach**2)**(-.5*(gamma+1.)/(gamma-1.))

def Sigma_Mach(Mach, gamma=defg._gamma):
    return (2./(gamma+1.)*(1.+.5*(gamma-1)*Mach**2))**(.5*(gamma+1.)/(gamma-1.))/Mach

def Mach_Sigma(sigma, Mach=2., gamma=defg._gamma):
	if np.size(Mach)!=1:
		Minit = np.where(Mach<=1., __MachSub_sigma(sigma, gamma), __MachSup_sigma(sigma, gamma))
	else:
		Minit=Mach
	def sigma_of_mach(m):
		return Sigma_Mach(m, gamma)-sigma
	result = newton(sigma_of_mach, Minit)
	return result #if np.size(result)!=1 else np.asscalar(result)

def MachSub_Sigma(sigma, gamma=defg._gamma):
	# initial guess
	Mach = __MachSub_sigma(sigma, gamma)
	return Mach_Sigma(sigma, Mach, gamma)

def MachSup_Sigma(sigma, gamma=defg._gamma):
	# initial guess
	#cg = (gamma+1.)/(gamma-1.)
	#Mach = np.sqrt((sigma*cg**(.5*cg))**(2./(cg-1.))-cg)
	Mach = __MachSup_sigma(sigma, gamma)
	return Mach_Sigma(sigma, Mach, gamma)




""" 
	The ``riemann`` module
	=========================
 
	Provides class and function for Riemann solvers for 1D unsteady flows
 
	:Example:
 
    >>> import aerokit.aero.Isentropic as Is
    >>> Is.TiTs_Mach(1.)
    1.2
    >>> Is.TiTs_Mach(2., gamma=1.6)
    2.2
 
    Available functions
    -------------------
 
    Blabla
  
"""

import math
import numpy     as np
from scipy.optimize import fsolve
import aerokit.aero.ShockWave  as sw
import aerokit.aero.unsteady1D as uq

# ===============================================================
# implementation of RIEMANN_PB class

class riemann_pb():
	"""
		Class riemann_pb for unsteady 1D Euler

		attributes:
			_qL, _qR : 
			_pstar
			_ustar
			_qstarL
			_qstarR
			_waves[5]

	"""
	def __init__(self, qL, qR):
		self._qL    = qL.copy()
		self._qR    = qR.copy()
		self._waves = 5*[0]
		# define _pstar
		self._solve()
		self._ustar    = self._ustar_from_pstar(self._pstar)
		self._waves[2] = self._ustar
		#
		# finalize qstarL state
		if (self._pstar > self._qL.p):
			_rho = self._qL.rho_through_shock(self._pstar)
			_wsh = (self._qL.massflow()-_rho*self._ustar)/(self._qL.rho-_rho)
			self._waves[0] = _wsh
			self._waves[1] = _wsh
			self._qstarL = uq.unsteady_state(rho=_rho, u=self._ustar, p=self._pstar, gamma=self._qL._gamma)
		else:
			_rho = self._qL.rho_through_isentropic(self._pstar)
			self._qstarL = uq.unsteady_state(rho=_rho, u=self._ustar, p=self._pstar, gamma=self._qL._gamma)
			self._waves[0] = self._qL.u  - self._qL.asound()
			self._waves[1] = self._ustar - self._qstarL.asound()
		#
		# finalize qstarR state
		if (self._pstar > self._qR.p):
			_rho = self._qR.rho_through_shock(self._pstar)
			_wsh = (self._qR.massflow()-_rho*self._ustar)/(self._qR.rho-_rho)
			self._waves[3] = _wsh
			self._waves[4] = _wsh
			self._qstarR = uq.unsteady_state(rho=_rho, u=self._ustar, p=self._pstar, gamma=self._qR._gamma)
		else:
			_rho = self._qR.rho_through_isentropic(self._pstar)
			self._qstarR = uq.unsteady_state(rho=_rho, u=self._ustar, p=self._pstar, gamma=self._qR._gamma)
			self._waves[3] = self._qR.u  + self._qR.asound()
			self._waves[4] = self._ustar + self._qstarR.asound()

	def __repr__(self):
		return "(rho, u, p)_L : (%s, %s, %s)\n" % (self._qL.rho, self._qL.u, self._qL.p) + \
			   "(rho, u, p)_R : (%s, %s, %s)"   % (self._qR.rho, self._qR.u, self._qR.p)

	def qsol(self, xot):
		"""
			xot: x/t numpy array
		"""
		# left uniform part (initialization)
		rho = self._qL.rho * np.ones(len(xot))
		u   = self._qL.u   * np.ones(len(xot))
		p   = self._qL.p   * np.ones(len(xot))
		# left star uniform part
		i = np.logical_and((xot >= self._waves[1]), (xot < self._waves[2]))
		rho[i] = self._qstarL.rho
		u[i]   = self._qstarL.u
		p[i]   = self._qstarL.p
		# right star uniform part
		i = np.logical_and((xot >= self._waves[2]), (xot < self._waves[3]))
		rho[i] = self._qstarR.rho
		u[i]   = self._qstarR.u
		p[i]   = self._qstarR.p
		# right uniform part
		i = (xot >= self._waves[4])
		rho[i] = self._qR.rho
		u[i]   = self._qR.u
		p[i]   = self._qR.p
		# if left expansion
		if (self._pstar < self._qL.p):
			i = np.logical_and((xot > self._waves[0]), (xot < self._waves[1]))
			gam = self._qL._gamma
			gp1 = gam+1.
			gm1 = gam-1.
			u[i]   = gm1/gp1*(self._qL.u + 2./gm1*(self._qL.asound()+xot[i]))
			p[i]   = self._qL.p * ( (u[i] - xot[i])/self._qL.asound() )**(2.*gam/gm1)
			rho[i] = self._qL.rho_through_isentropic(p[i])
		# if right expansion
		if (self._pstar < self._qR.p):
			i = np.logical_and((xot > self._waves[3]), (xot < self._waves[4]))
			gam = self._qR._gamma
			gp1 = gam+1.
			gm1 = gam-1.
			u[i]   = gm1/gp1*(self._qR.u + 2./gm1*(-self._qR.asound()+xot[i]))
			p[i]   = self._qR.p * ( (xot[i] - u[i] )/self._qR.asound() )**(2.*gam/gm1)
			rho[i] = self._qR.rho_through_isentropic(p[i])
		return uq.unsteady_state(rho, u, p, np.where(xot<0., self._qL._gamma, self._qR._gamma))

	def left_fastest(self):
		return self._waves[0]

	def right_fastest(self):
		return self._waves[4]

	def ustar(self):
		return self._ustar

	def pstar(self):
		return self._pstar

	def qstarL(self):
		return self._qstarL

	def qstarR(self):
		return self._qstarR

	def _delta_uL(self, p):
		"""left contribution to delta u in Riemann problem"""
		if (p > self._qL.p):
			_delta = self._qL.delta_u_shock(p)
		else:
			_delta = self._qL.delta_u_expansion(p)
		return _delta

	def _delta_uR(self, p):
		"""right contribution to delta u in Riemann problem"""
		if (p > self._qR.p):
			_delta = self._qR.delta_u_shock(p)
		else:
			_delta = self._qR.delta_u_expansion(p)
		return _delta

	def _ustar_from_pstar(self, p):
		return .5*(self._qR.u + self._qL.u + self._delta_uR(p) - self._delta_uL(p)) 

	def _pstar_estimate_expansion(self):
		"""exact solution for 2 expansion waves (for uniform gamma only)

			..note: from Toro, page 128
		"""
		gam   = .5*(self._qL._gamma+self._qR._gamma)
		gmusd = .5*(gam-1.)
		return ((self._qL.asound()+self._qR.asound()
			    -gmusd*(self._qR.u - self._qL.u)) / 
				( self._qL.asound()/self._qL.p**(gmusd/gam)
				+ self._qR.asound()/self._qR.p**(gmusd/gam) ))**(gam/gmusd)

	def _solve(self):
		def du(p):
			return self._delta_uL(p) + self._delta_uR(p) + self._qR.u - self._qL.u
		self._pstar = fsolve(du, self._pstar_estimate_expansion(), xtol=1e-10)
		return




# coding: utf-8

# <img align="right" src="./Downloads/cycle turbofan.tiff" width=400px>
# <h1 align="left" style="font-size:60px">Construction d'un cycle de Turbofan</h1>

# <h2>1. Importation des librairies python</h2>

# In[1]:


import numpy as np
import aerokit.aero.Isentropic as Is

# get_ipython().magic(u'matplotlib inline')
# get_ipython().magic(u'xmode verbose')

# ## 2. Introduction à la programmation objet python, définition de l'objet Gaz (gaz parfait)

# On propose de coder la classe $\textbf{Gaz}$, qui a pour arguments d'instance, $\gamma$, $r$, $P_k$ et une méthode get_cp(self).<br>
# Pour utiliser un objet en python il faut d'abords déclarer une classe comme en JAVA. 
# Au lieu de "$\textbf{def}$ function_name(args):" en python, on doit déclarer "$\textbf{class}$ class_name:"<br>
# Les attributs se déclarent comme des variable en python dans la class.<br>
# Pour déclarer une méthode on utilise "$\textbf{def}$ nom_de_methode(self, args):"<br>
# La méthode "$\textbf{__init__(self, args):}$" correspond au constructeur en JAVA.<br>
# L'argument "$\textbf{self}$" représente l'instance de l'objet (pointeur vers l'objet en memoire). On a donc 2 types d'arguments, les attributs qui sont communs à chaque objet (ex: M_0 = 0.8) et les attributs qui sont proprent à chaque objet (ex : self.gamma = 1.4).

# In[2]:


class Gaz:
    
    def __init__(self, gamma=1.4, r=287., P_k=0):
        self.gamma = gamma
        self.r = r
        self.P_k = P_k
        self.gsgmu = gamma / (gamma - 1.)
        self.gmusg = (gamma - 1.) / gamma
        
    def get_cp(self):
        return (self.r * self.gamma) / (self.gamma - 1.)


# ## 3. Construction du cycle de l'étage 1 à 4

# On construit maintenant une classe $\textbf{cycle_1_to_4}$ qui représentera un turbofan de l'étage 1 jusqu'à la sortie de chambre de combustion. Cette classe prend comme arguments : $P_0$, $T_0$, $M_0$, ainsi que les différents $\eta$ et $\xi$. Il y aura une méthode pour le calcule de chaque étage.<br><br>
# $\textbf{Rappels de cours :}$
# <ul>
#     <li>Étage 1 :</li>
#           <ul>
#                <li>Grandeurs totales : $$T_{i}=T\cdot \left(1 + \frac{\gamma -1}{2}\cdot M^2\right) \quad \& \quad P_{i}=P\cdot \left(1 + \frac{\gamma -1}{2}\cdot M^2\right)^{\frac{\gamma}{\gamma -1}}$$</li>
#           </ul>
#      <li>Étage 2 :</li>
#           <ul>
#                <li>Notion de perte de charge : $$P_{i2}=\xi \cdot P_{i1} \quad \& \quad T_{i2}=T_{i1}$$</li>
#           </ul>
#      <li>Étage 3 :</li>
#             <ul>
#                <li>Compression polytropique : $$\pi_c=\frac{P_{i3}}{P_{i2}}=\frac{T_{i3}}{T_{i2}}^{\frac{\gamma \cdot \eta_{pol}}{\gamma - 1}} \quad \& \quad \dot{w}_u=\dot{m}_c \cdot c_p \cdot \left(T_{i3} - T_{i2}\right)$$</li>
#           </ul>
#      <li>Étage 4:
#           <ul>
#                <li>Chambre de combustion : $$\alpha =\frac{\dot{m}_k}{\dot{m}} \quad \& \quad \dot{m}_c c^*_p\left(1+\alpha\right) T_{i4} - \dot{m}_c c_p T_{i3}=\eta_{comb} \dot{m}_k P_k \quad \& \quad P_{i4}=\xi \cdot P_{i3}$$</li>
#           </ul>
#      </li>
# </ul>

# In[3]:


class cycle_1_to_4:
    
    P0 = 227.00e2
    T0 = 217.00
    M0 = 0.8
    
    xi_e = 0.97
    eta_c = 0.90
    eta_f = 0.89
    eta_comb = 0.99
    xi_cc = 0.95
    eta_m = 0.98
    eta_turb = 0.88
    xi_tuy = 0.97
    
    def __init__(self, lambda_, pi_c, Tt4, m, g, g_fuel):
        self.lambda_ = lambda_
        self.pi_c = pi_c
        self.Tt4 = Tt4
        self.m = m
        self.g = g
        self.g_fuel = g_fuel
        self.current_stage_corps = 0
        
    def show_attributes(self):
        for attr in dir(self):
            if (not callable(getattr(self, attr)) and not attr.startswith('__')) :
                print("%s = %r" % (attr, getattr(self, attr)))
    
    def InitialValues(self):
        self.Pt0 = self.P0 * Is.PiPs_Mach(Mach=self.M0, gamma=self.g.gamma)
        self.Tt0 = self.T0 * Is.TiTs_Mach(Mach=self.M0, gamma=self.g.gamma)
        self.V0 = self.M0 * np.sqrt(self.g.gamma * self.g.r * self.T0)
        self.current_stage_corps = 1
        
    def stage_2(self):
        if self.current_stage_corps == 1:
            self.Tt2 = self.Tt0
            self.Pt2 = self.Pt0 * self.xi_e
            self.current_stage_corps = 2
        else:
            print ("Il faut initialiser les valeurs d'entrées à l'aide de la méthode InitialValues")
        
    def stage_3(self):
        if self.current_stage_corps == 2:
            self.m_c = self.m / (self.lambda_ + 1.)
            self.Tt3 = self.Tt2 * self.pi_c**(self.g.gmusg / self.eta_c)
            self.Pt3 = self.Pt2 * self.pi_c
            self.wu_c = self.m_c * self.g.get_cp() * (self.Tt3 - self.Tt2)
            self.current_stage_corps = 3
            
    def stage_4(self):
        if self.current_stage_corps == 3:
            self.m_k = self.m_c * (((self.g_fuel.get_cp() * self.Tt4) - (self.g.get_cp() * self.Tt3))/((self.eta_comb * self.g_fuel.P_k) - (self.g_fuel.get_cp() * self.Tt4)))
            self.alpha = self.m_k / self.m_c
            self.Pt4 = self.Pt3 * self.xi_cc
            self.current_stage_corps = 4
            
    def calculs_1_to_4(self):
        if self.current_stage_corps == 0:
            self.InitialValues()
            self.stage_2()
            self.stage_3()
            self.stage_4()
        if self.current_stage_corps == 1:
            self.stage_2()
            self.stage_3()
            self.stage_4()
        if self.current_stage_corps == 2:
            self.stage_3()
            self.stage_4()
        if self.current_stage_corps == 3:
            self.stage_4()
        


# ## 4. Construction du cycle l'étage 4 à la fin<br>
# On construit en suite une nouvelle classe $\textbf{cycle_5_to_9}$ qui hérite de la classe $\textbf{cycle_1_to_4}$ en déclarant "$\textbf{class}$ cycle_5_to_9(cycle_1_to_4):". Cela veut dire que la nouvelle classe héritera des attributs et des méthodes de la classe $\textbf{cycle_1_to_4}$.<br>
# Nous proposons 2 méthodes pour le calcul de la fin du cycle, une à $\pi_{fan}$ fixé et une autre à $\pi_{fan}$ calculé en renvoyant un pourcentage de la puissance récupérée dans la turbine BP vers le fan.<br><br>
# $\textbf{Rappels de cours :}$
# <ul>
#     <li>Turbines : $$\pi_t=\frac{P_{i4}}{P_{i5}}=\left(\frac{T_{i4}}{T_{i5}}\right)^{\frac{\gamma}{\eta_{pol}\left(\gamma - 1\right)}} \quad \& \quad \dot{w}_u=\dot{m}_c \left(1+\alpha \right) c^*_p \left(T_{i5} - T_{i4}\right)$$</li>
#     <li>Tuyères : $$T_{i9}\simeq T_{i5} \quad \& \quad P_{i9}=\xi \cdot P_{i5}$$</li>
#     <li>Poussée : $$F=\dot{m} \left(V_9 - V_0\right)$$</li>
#     <li>Puissance propulsive : $$P_F=F \cdot V_0$$</li>
#     <li>Variation d'énergie cinétique des gaz : $$\Delta E_c=\frac{1}{2} {m}_c \left(\left(1+\alpha\right)V^2_9 - V^2_0\right) + \frac{1}{2} {m}_f \left(V^2_{19} - V^2_{0}\right)$$</li>
#     <li>Puissance thermique de combustion : $$\dot{Q}_f=\dot{m}_k \cdot P_k$$</li>
#     <li>Rendements : $$\eta_T=\frac{\Delta E_c}{\dot{Q}_f} \quad \& \quad \eta_P=\frac{P_F}{\Delta E_c} \quad \& \quad \eta = \eta_T \cdot \eta_P$$</li>
#     <li>Grandeurs spécifique : $$F_{spe}=\frac{F}{\dot{m}}\quad \& \quad C_s=\frac{\dot{m}_k}{F}$$</li>
# </ul>

# ### 4.1. Méthode 1 : à $\pi_f$ fixé

# On déclare une classe qui hérite de la classe précedente

# In[4]:


class cycle_taux_fan_fixe(cycle_1_to_4):
    
    def __init__(self, lambda_, pi_c, Tt4, m, g, g_fuel, pi_f):
        self.lambda_ = lambda_
        self.pi_c = pi_c
        self.Tt4 = Tt4
        self.m = m
        self.g = g
        self.g_fuel = g_fuel
        self.pi_f = pi_f
        self.current_stage_corps = 0
        self.current_stage_fan = 0
        self.calculs_1_to_4()
        
    def stage_13(self):
        if self.current_stage_corps >= 2:
            self.m_f = self.m * self.lambda_ / (self.lambda_ + 1.)
            self.Tt13 = self.Tt2 * self.pi_f**(self.g.gmusg / self.eta_c)
            self.Pt13 = self.Pt2 * self.pi_f
            self.wu_f = self.m_f * self.g.get_cp() * (self.Tt13 - self.Tt2)
            self.current_stage_fan = 13
            
    def stage_19(self):
        if self.current_stage_fan == 13:
            self.Pt19 = self.Pt13 * self.xi_tuy
            self.Tt19 = self.Tt13
            self.P19 = self.P0
            self.M19 = Is.Mach_PiPs(self.Pt19/self.P19, self.g.gamma)
            self.V19 = Is.Velocity_MachTi(self.M19, self.Tt19, self.g.r, self.g.gamma)
            self.F_f = self.m_f * (self.V19 - self.V0)
            self.current_stage_fan = 19
            
    def stage_5(self):
        if self.current_stage_corps == 4 and self.current_stage_fan >= 13:
            self.Tt5 = self.Tt4 - ((self.wu_f + self.wu_c)/(self.m_c * (1. + self.alpha) * self.g_fuel.get_cp() * self.eta_m))
            self.pi_t = (self.Tt4 / self.Tt5)**(self.g_fuel.gsgmu / self.eta_turb)
            self.Pt5 = self.Pt4 / self.pi_t
            self.current_stage_corps = 5
            
    def stage_9(self):
        if self.current_stage_corps == 5:
            self.Tt9 = self.Tt5
            self.Pt9 = self.Pt5 * self.xi_tuy
            self.P9 = self.P0
            self.M9 = Is.Mach_PiPs(self.Pt9/self.P9, self.g_fuel.gamma)
            self.V9 = Is.Velocity_MachTi(self.M9, self.Tt9, self.g_fuel.r, self.g_fuel.gamma)
            self.F_c = self.m_c * (self.V9 - self.V0)
            self.current_stage_corps = 9
            
    def perfo(self):
        if (self.current_stage_corps == 9 and self.current_stage_fan == 19):
            self.F = self.F_f + self.F_c
            self.F_spe = self.F / self.m
            self.mk_spe = (self.m_k / self.F)*10e6
            Qf = self.g_fuel.P_k * self.m_k
            delta_E = .5*(self.m_c*((1. + self.alpha)*self.V9**2 - self.V0**2) + self.m_f*(self.V19**2 - self.V0**2))
            self.eta_th = delta_E / Qf
            self.eta_prop = (self.F * self.V0) / delta_E
            self.eta = self.eta_th * self.eta_prop
            
    def show_perfo(self):
        if self.current_stage_fan == 19 and self.current_stage_corps == 9:
            print("%s\t\t\t%.0f\t%s" % ("Poussée :", self.F, "N"))
            print("%s\t\t%.2f\t%s" % ("Poussée spécifique :", self.F_spe, "N.s/kg"))
            print("%s\t%.2f\t%s" % ("Consommation spécifique :", self.mk_spe, "g/kN/s"))
            print("%s\t\t%.3f\t%s" % ("Rendement thermique :", self.eta_th, ""))
            print("%s\t\t%.3f\t%s" % ("Rendement propulsif :", self.eta_prop, ""))
            print("%s\t\t%.3f\t%s" % ("Rendement global :", self.eta, ""))
    
    def calculs_5_to_9(self):
        if self.current_stage_fan == 0:
            self.stage_13()
            self.stage_19()
            self.stage_5()
            self.stage_9()
            self.perfo()
        if self.current_stage_fan == 13:
            self.stage_19()
            self.stage_5()
            self.stage_9()
            self.perfo()
        if self.current_stage_fan == 19:
            self.stage_5()
            self.stage_9()
            self.perfo()
        if self.current_stage_corps == 5:
            self.stage_9()
            self.perfo()
        if self.current_stage_fan == 19 and self.current_stage_corps == 9:
            self.perfo()
                               


# ### 4.1. Méthode 2 : à $\pi_f$ calculé

# In[9]:


class cycle_taux_fan_calcule(cycle_1_to_4):
    
    def __init__(self, lambda_, pi_c, Tt4, m, g, g_fuel, taux_meca=.5):
        self.lambda_ = lambda_
        self.pi_c = pi_c
        self.Tt4 = Tt4
        self.m = m
        self.g = g
        self.g_fuel = g_fuel
        self.taux_meca = taux_meca
        self.current_stage_corps = 0
        self.current_stage_fan = 0
        self.calculs_1_to_4()
        
    def stage_4_etoile(self):
        if self.current_stage_corps == 4:
            self.Tt4_etoile = self.Tt4 - ((self.wu_c)/(self.m_c * (1. + self.alpha) * self.g_fuel.get_cp() * self.eta_m))
            self.pi_t_c = (self.Tt4 / self.Tt4_etoile)**(self.g_fuel.gsgmu / self.eta_turb)
            self.Pt4_etoile = self.Pt4 / self.pi_t_c
            self.current_stage_corps = 5
            
    def stage_5(self):
        if self.current_stage_corps == 5:
            P5 = self.P0
            T5 = self.Tt4_etoile * (P5 / self.Pt4_etoile)**self.g_fuel.gmusg
            w_uf_theo = self.m_c * (1. + self.alpha) * self.g_fuel.get_cp() * (T5 - self.Tt4_etoile)
            self.wu_fan = w_uf_theo * self.taux_meca
            self.Tt5 = (self.wu_fan / (self.g_fuel.get_cp() * self.m_c * (1. + self.alpha))) + self.Tt4_etoile
            self.pi_t_f = (self.Tt4_etoile / self.Tt5)**(self.g_fuel.gsgmu / self.eta_turb)
            self.Pt5 = self.Pt4_etoile / self.pi_t_f
            self.current_stage_corps = 6
    
    def stage_9(self):
        if self.current_stage_corps == 6:
            self.Tt9 = self.Tt5
            self.Pt9 = self.Pt5 * self.xi_tuy
            self.P9 = self.P0
            self.M9 = Is.Mach_PiPs(self.Pt9/self.P9, self.g_fuel.gamma)
            self.V9 = Is.Velocity_MachTi(self.M9, self.Tt9, self.g_fuel.r, self.g_fuel.gamma)
            print("V9  cla", self.V9, self.M9)
            self.F_c = self.m_c * (self.V9 - self.V0)
            self.current_stage_corps = 9
            
    def stage_13(self):
        if self.current_stage_corps >= 6:
            self.m_f = (self.lambda_ * self.m) / (self.lambda_ + 1.)
            self.Tt13 = -self.wu_fan * self.eta_m /(self.m_f * self.g.get_cp()) + self.Tt2
            self.pi_f = (self.Tt13 / self.Tt2)**(self.eta_f * self.g.gsgmu)
            self.Pt13 = self.Pt2 * self.pi_f
            self.current_stage_fan = 13
            
    def stage_19(self):
        if self.current_stage_fan == 13:
            self.Pt19 = self.Pt13 * self.xi_tuy
            self.Tt19 = self.Tt13
            self.P19 = self.P0
            self.M19 = Is.Mach_PiPs(self.Pt19/self.P19, self.g.gamma)
            self.V19 = Is.Velocity_MachTi(self.M19, self.Tt19, self.g.r, self.g.gamma)
            print("V19 cla", self.V19)
            self.F_f = self.m_f * (self.V19 - self.V0)
            self.current_stage_fan = 19
            
    def perfo(self):
        if (self.current_stage_corps == 9 and self.current_stage_fan == 19):
            self.F = self.F_f + self.F_c
            self.F_spe = self.F / self.m
            self.mk_spe = (self.m_k / self.F)*10e6
            Qf = self.g_fuel.P_k * self.m_k
            delta_E = .5*(self.m_c*((1. + self.alpha)*self.V9**2 - self.V0**2) + self.m_f*(self.V19**2 - self.V0**2))
            self.eta_th = delta_E / Qf
            self.eta_prop = (self.F * self.V0) / delta_E
            self.eta = self.eta_th * self.eta_prop
            return self.F, self.F_spe, self.mk_spe, self.eta_th, self.eta_prop, self.eta
            
    def show_perfo(self):
        if self.current_stage_fan == 19 and self.current_stage_corps == 9:
            print("%s\t\t\t%.0f\t%s" % ("Poussée :", self.F, "N"))
            print("%s\t\t%.2f\t%s" % ("Poussée spécifique :", self.F_spe, "N.s/kg"))
            print("%s\t%.2f\t%s" % ("Consommation spécifique :", self.mk_spe, "g/kN/s"))
            print("%s\t\t%.3f\t%s" % ("Rendement thermique :", self.eta_th, ""))
            print("%s\t\t%.3f\t%s" % ("Rendement propulsif :", self.eta_prop, ""))
            print("%s\t\t%.3f\t%s" % ("Rendement global :", self.eta, ""))
            
    
    def calculs_5_to_9(self):
        if self.current_stage_corps == 4:
            self.stage_4_etoile()
            self.stage_5()
            self.stage_9()
            self.stage_13()
            self.stage_19()
            self.perfo()
        if self.current_stage_corps == 5:
            self.stage_5()
            self.stage_9()
            self.stage_13()
            self.stage_19()
            self.perfo()
        if self.current_stage_corps == 6:
            self.stage_9()
            self.stage_13()
            self.stage_19()
            self.perfo()
        if self.current_stage_corps == 9:
            self.stage_13()
            self.stage_19()
            self.perfo()
        if self.current_stage_fan == 13:
            self.stage_19()
            self.perfo()
        if self.current_stage_fan == 19 and self.current_stage_corps == 9:
            self.perfo()


# ## 5. Résultats

# In[13]:


# g = Gaz()
# g_fuel = Gaz(1.33,291.6,42800.e3)
# c1 = cycle_taux_fan_fixe(6.1,32.8,1410.,180.,g,g_fuel,1.6)
# c2 = cycle_taux_fan_calcule(11,41,1620.,230.,g,g_fuel,0.58)
# c1.calculs_5_to_9()
# c2.calculs_5_to_9()
# print("CFM56 :")
# c1.show_perfo()
# print "LEAP 1A :"
# c2.show_perfo()




"""@package Fanno
@brief functions to compute Fanno transformation in subsonic/supersonic ducts
"""

import numpy as np

def maxFparam_Mach(Mach, gamma=1.4):
    """ computes maximum Fanno parameter for a given Mach number state
    This maximum value makes the flow reach a sonic state
    \f$
      \left(\frac{1 - M^2}{\gamma M^2}\right) + \left(\frac{\gamma + 1}{2\gamma}\right)\ln\left[\frac{M^2}{\left(\frac{2}{\gamma + 1}\right)\left(1 + \frac{\gamma - 1}{2}M^2\right)}\right]
    \f$
    """
    m2 = np.square(Mach)
    return (1-m2)/(gamma*m2) + (gamma+1)/(2*gamma)*np.log(m2*(gamma+1)/2/(1+(gamma-1)/2*m2))

# --- ratio to critical/choking state ---

def Ps_Pscri(Mach, gamma=1.4):
    """ computes Ps/Ps_cri ratio from Mach number
    """
    return 1./(Mach*np.sqrt(2/(gamma+1)*(1+(gamma-1)/2*np.square(Mach))))

def Ts_Tscri(Mach, gamma=1.4):
    """ computes Ts/Ts_cri ratio from Mach number
    """
    return (gamma+1)/2/(1+(gamma-1)/2*np.square(Mach))
    
def Rho_Rhocri(Mach, gamma=1.4):
    """ computes rho/rho_cri ratio from Mach number
    """
    return 1./Mach*np.sqrt(2/(gamma+1)*(1+(gamma-1)/2*np.square(Mach)))
    
def Pi_Picri(Mach, gamma=1.4):
    """ computes Pi/Pi_cri ratio from Mach number
    """
    return 1./Mach*(2/(gamma+1)*(1+(gamma-1)/2*np.square(Mach)))**((gamma+1)/2/(gamma-1))

def V_Vcri(Mach, gamma=1.4):
    """ computes V/a_cri ratio from Mach number
    """
    return Mach / np.sqrt(2/(gamma+1)*(1+(gamma-1)/2*np.square(Mach)))

def NormdS(Mach, gamma=1.4):
    """ computes delta S / Cp up to critical point
    """
    return np.log(Mach**((gamma-1)/gamma) * (2/(gamma+1)*(1+(gamma-1)/2*np.square(Mach)))**(-(gamma+1)/2/gamma))





"""
    The ``nozzle`` module
    =========================
 
    Provides specific functions for computation of 1D defintion of nozzles via section law
 
    :Example:
 
    >>> import aerokit.aero.nozzle as nz
    >>> Is.TiTs_Mach(1.)
    1.2
    >>> Is.TiTs_Mach(2., gamma=1.6)
    2.2
 
    Available functions
    -------------------
 
	.. note:: Specific heat ratio `gamma` is defined only using aerokit.common.defaultgas module
"""


import math
import numpy as np
from aerokit.common import defaultgas as defg # relative import is deprecated by doctest
from aerokit.aero   import Isentropic as Is
from aerokit.aero   import MassFlow   as mf
from aerokit.aero   import ShockWave  as sw

# === NPR computation from As/Ac definition of nozzle ===

def NPR_choked_subsonic(AsAc):
	return Is.PiPs_Mach(mf.MachSub_Sigma(AsAc))

def NPR_choked_supersonic(AsAc):
	return Is.PiPs_Mach(mf.MachSup_Sigma(AsAc))

def NPR_shock_at_exit(AsAc):
	Msup  = mf.MachSup_Sigma(AsAc)
	Msh   = sw.downstream_Mn(Msup)
	return Is.PiPs_Mach(Msh) / sw.Pi_ratio(Msup)

def _NPR_Ms_list(AsAc):
	"""
    	Computes all NPR limits and associated exit Mach number

		internal function
 
		:param AsAc:  ratio of section at exit over throat
		:return:      result NPR and Mach numbers
 
 		:Example:

		>>> import aerokit.aero.MassFlow as mf ; mf.Sigma_Mach(Is.Mach_PiPs(np.array(_NPR_Ms_list(2.)[:3:2])))
		array([ 2.,  2.])

		.. seealso:: NPR_choked_subsonic(), NPR_choked_supersonic(), NPR_shock_at_exit()
		.. note:: available for scalar or array (numpy) computations
    """
	Msub  = mf.MachSub_Sigma(AsAc)
	NPR0  = Is.PiPs_Mach(Msub)
	Msup  = mf.MachSup_Sigma(AsAc)
	Msh   = sw.downstream_Mn(Msup)
	NPRsw = Is.PiPs_Mach(Msh) / sw.Pi_ratio(Msup)
	NPR1  = Is.PiPs_Mach(Msup)
	return NPR0, NPRsw, NPR1, Msub, Msh, Msup

def Ms_from_AsAc_NPR(AsAc, NPR):
	"""
    	Computes Mach number at exit of a nozzle given As/Ac and NPR

		This method checks the NPR to define regime and computes Mach number at exit
 
		:param AsAc:  ratio of section at exit over throat
        :param NPR:   ratio of total pressure at inlet over 'expected' static pressure at exit
		:return:      result Mach number at exit
 
 		:Example:

		>>> print round(Ms_from_AsAc_NPR(2.636, 1.5), 8) # case with shock in diffuser
		0.32586574

		.. seealso:: 
		.. note:: NOT available for array (numpy) computations
    """	
	NPR0, NPRsw, NPR1, Msub, Msh, Msup = _NPR_Ms_list(AsAc)
	if (NPR < NPR0):
		Ms = Is.Mach_PiPs(NPR)
	elif (NPR > NPRsw): 
		Ms = Msup
	else:
		gmu = defg._gamma-1.
		K   = NPR/AsAc/((defg._gamma+1.)/2)**((defg._gamma+1.)/2/gmu)
		Ms  = np.sqrt((np.sqrt(1.+2.*gmu*K*K)-1.)/gmu)
	return Ms

def Madapt_from_AsAc_NPR(AsAc, NPR):
	"""
    	Computes Mach number for pressure adapted flow of a nozzle given As/Ac and NPR

		This method checks the NPR to define regime and computes Mach number in jet. The switch between 
		overexpanded jet and underexpanded jet is 
 
		:param AsAc:  ratio of section at exit over throat
        :param NPR:   ratio of total pressure at inlet over 'expected' static pressure at exit
		:return:      result Mach number at exit
 
 		:Example:

		>>> print round(Ms_from_AsAc_NPR(2.636, 1.5), 8) # case with shock in diffuser
		0.32586574

		.. seealso:: 
		.. note:: NOT available for array (numpy) computations
    """	
	NPR0, NPRsw, NPR1, Msub, Msh, Msup = _NPR_Ms_list(AsAc)
	if (NPR < NPR0):
		Ms = Is.Mach_PiPs(NPR)
	elif (NPR > NPR1): # under expanded flow
		Ms = Is.Mach_PiPs(NPR)
	elif (NPR > NPRsw): # shock wave in jet
		Ms = sw.downstreamMach_Mach_ShockPsratio(Msup, NPR1/NPR)
	else:
		gmu = defg._gamma-1.
		K   = NPR/AsAc/((defg._gamma+1.)/2)**((defg._gamma+1.)/2/gmu)
		Ms  = np.sqrt((np.sqrt(1.+2.*gmu*K*K)-1.)/gmu)
	return Ms



class nozzle():

	def __init__(self, x, section, AsoAc=None, gamma=1.4):

		self.gamma   = gamma
		self.x       = x
		if AsoAc:
			self.AsoAc = AsoAc
		else:
			self.AsoAc = section[-1] / np.min(section)
		self.AxoAc   = section * self.AsoAc / section[-1]
		self.ithroat = np.abs(self.AxoAc).argmin() # abs not necessary but ensure conversion to numpy array
		self.NPR0, self.NPRsw, self.NPR1, self.Msub, self.Msh, self.Msup = _NPR_Ms_list(self.AsoAc)
		#print self.NPR0, self.NPRsw, self.NPR1

	def set_NPR(self, NPR):
		self._Pt = np.ones_like(self.AxoAc)
		if NPR < self.NPR0:
			_Ms = Is.Mach_PiPs(NPR, gamma=self.gamma)
			self._M  = mf.MachSub_Sigma(self.AxoAc/self.AsoAc*mf.Sigma_Mach(_Ms), gamma=self.gamma)
			self._Ps = self._Pt/Is.PiPs_Mach(self._M, gamma=self.gamma)
		else:
			self._M = np.ones_like(self.AxoAc)
			self._M[:self.ithroat+1]   = mf.MachSub_Sigma(self.AxoAc[:self.ithroat+1],   gamma=self.gamma)
			self._M[self.ithroat+1:] = mf.MachSup_Sigma(self.AxoAc[self.ithroat+1:], gamma=self.gamma)
			if NPR < self.NPRsw:
				# analytical solution for Ms, losses and upstream Mach number of shock wave
				Ms     = Ms_from_AsAc_NPR(self.AsoAc, NPR)
				Ptloss = Is.PiPs_Mach(Ms)/NPR
				Msh    = sw.Mn_Pi_ratio(Ptloss)
				# redefine curves starting from 'ish' index (closest value of Msh in supersonic flow)
				ish    = np.abs(self._M-Msh).argmin()
				self._M[ish:] = mf.MachSub_Sigma(self.AxoAc[ish:]*mf.Sigma_Mach(Ms)/self.AsoAc)
				self._Pt[ish:] = Ptloss
			self._Ps = self._Pt/Is.PiPs_Mach(self._M)


	def Mach(self):
		return self._M

	def Ps(self):
		return self._Ps

	def Ptot(self):
		return self._Pt


# ===============================================================
# automatic testing

if __name__ == "__main__":
    import doctest
    doctest.testmod()


# coding: utf-8

# Grandeurs d'entrée
P0 = 227.00e2
T0 = 217.00
M0 = 0.8

# Paramètres
xi_e = 0.97
eta_c = 0.90
eta_f = 0.89
eta_comb = 0.99
xi_cc = 0.95
eta_m = 0.98
eta_turb = 0.88
xi_tuy = 0.97

P_k = 42800.00e3

# Gaz parfait
r = 287.00
r_ = 291.60
gamma = 1.40
gamma_ = 1.33

# Calculs
def calculs(lambda_, pi_c, Tt4, pi_f, m):
    V0 = M0 * (gamma * r * T0)**(1/2)
    # Calculs grandeurs totales entrée
    Pt0 = P0 * (1 + ((gamma - 1) / 2) * M0**2)**(gamma/(gamma - 1))
    Tt0 = T0 * (1 + ((gamma - 1) / 2) * M0**2)
    # Calculs plan 2
    Tt2 = Tt0
    Pt2 = Pt0*xi_e
    # Gaz parfait
    cp = (r * gamma)/(gamma - 1)
    cp_ = (r_ * gamma_)/(gamma_ - 1)
    # Calculs fan
    m_f = (lambda_ * m) / (lambda_ + 1)
    Tt17 = Tt2 * pi_f**((gamma - 1)/(gamma * eta_f))
    Pt17 = Pt2 * pi_f
    w_uf = m_f * cp * (Tt17 - Tt2)
    # Calculs CHP
    m_c = m / (lambda_ + 1)
    Tt3 = Tt2 * pi_c**((gamma - 1)/(gamma * eta_c))
    Pt3 = Pt2 * pi_c
    w_uc = m_c * cp * (Tt3 - Tt2)
    # Calculs chambre de combustion
    m_k = m_c * (((cp_ * Tt4) - (cp * Tt3))/((eta_comb * P_k) - (cp_ * Tt4)))
    alpha = m_k / m_c
    Pt4 = Pt3 * xi_cc
    # Calculs turbines
    Tt5 = Tt4 - ((w_uf + w_uc)/(m_c * (1 + alpha) * cp_ * eta_m))
    pi_t = (Tt4 / Tt5)**(gamma_ / (eta_turb * (gamma_ - 1)))
    Pt5 = Pt4 / pi_t
    # Calculs tuyère corps
    Tt9 = Tt5
    Pt9 = Pt5 * xi_tuy
    P9 = P0
    M9 = (((Pt9/P9)**((gamma_ - 1)/gamma_ ) - 1) * (2/(gamma_ - 1)))**(1/2)
    T9 = Tt9 / (1 + ((gamma_ - 1)/2)*M9**2)
    A9 = (gamma_ * r_ * T9)**(1/2)
    V9 = M9 * A9
    Fc = m_c * (V9 - V0)
    # Calculs tuyère fan
    Pt19 = Pt17 * xi_tuy
    Tt19 = Tt17
    P19 = P0
    M19 = (((Pt19/P19)**((gamma - 1)/gamma ) - 1) * (2/(gamma - 1)))**(1/2)
    T19 = Tt19 / (1 + ((gamma - 1)/2)*M19**2)
    A19 = (gamma * r * T19)**(1/2)
    V19 = M19 * A19
    Ff = m_f * (V19 - V0)
    # Calculs performances
    F = Fc + Ff
    Fspe = F/m
    mk_spe = (m_k / F)*10e6
    Qf = P_k * m_k
    delta_E = (1/2)*(m_c*((1 + alpha)*V9**2 - V0**2) + m_f*(V19**2 - V0**2))
    eta_th = delta_E / Qf
    eta_prop = (F * V0) / delta_E
    eta = eta_th * eta_prop
    print(w_uc)
    print(w_uf)
    print(Tt5)
    print("F = ",format(F,'.0f'))
    print("Fspe = ", format(Fspe,'.0f'))
    print("conso spe = ", format(mk_spe,'.10f'))
    print("eta_th = ",format(eta_th,'.3f'))
    print("eta_prop = ",format(eta_prop, '.3f'))
    print("eta = ",format(eta, '.3f'))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aerokit',
 'aerokit.aero',
 'aerokit.aero.plot',
 'aerokit.common',
 'aerokit.engine']

package_data = \
{'': ['*']}

install_requires = \
['matplotlib>=3.3.3,<4.0.0', 'numpy>=1.19.4,<2.0.0', 'scipy>=1.5.4,<2.0.0']

setup_kwargs = {
    'name': 'aerokit',
    'version': '1.0.0',
    'description': 'Python tools for basic fluid mechanics computations',
    'long_description': None,
    'author': 'j.gressier',
    'author_email': 'jeremie.gressier@isae-supaero.fr',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
}


setup(**setup_kwargs)



"""
    The ``Isentropic`` module
    =========================
 
    Provides standart Stagnation to Static Ratios for Ideal gas
 
    :Example:
 
    >>> import aerokit.aero.Isentropic as Is
    >>> Is.TiTs_Mach(1.)
    1.2
    >>> Is.TiTs_Mach(2., gamma=1.6)
    2.2
 
    Available functions
    -------------------
 
    Provides Ti/Ts Pi/Ps ratios from Mach number and reversed functions.
    Specific heat ratio `gamma` is optionnal and can be specified in the functions itself
    or using aerokit.common.defaultgas module
 """

import math
import numpy as np
from aerokit.common import defaultgas as defg # relative import is deprecated by doctest

# ===============================================================
# implemented functions

def TiTs_Mach(Mach, gamma=defg._gamma):
    """
    	Computes Ti/Ts ratio from Mach number

		Long comment
 
		:param Mach:  local Mach number
        :param gamma: specific heat ratio, default from aerokit.common.defaultgas
		:return:      result Ti/Ts ratio
 
 		:Example:

		>>> TiTs_Mach(1.) # with default gamma 1.4
		1.2
		>>> TiTs_Mach(2., gamma=1.6)
		2.2

		.. seealso:: Mach_TiTs()
		.. note:: available for scalar or array (numpy) computations
    """	
    return 1.+.5*(gamma-1)*Mach**2

def PiPs_Mach(Mach, gamma=defg._gamma):
    """
    	Computes Pi/Ps ratio from Mach number

		Long comment
 
		:param Mach:  local Mach number
        :param gamma: specific heat ratio, default from aerokit.common.defaultgas
		:return:      result Pi/Ps ratio
 
 		:Example:

		>>> PiPs_Mach(1.) # with default gamma 1.4
		1.892929158737854
		>>> PiPs_Mach(2., gamma=1.6)
		8.187044460255244

		.. seealso:: Mach_PiPs()
		.. note:: available for scalar or array (numpy) computations
    """	
    return (1.+.5*(gamma-1.)*Mach**2)**(gamma/(gamma-1.))

def Mach_TiTs(TiTs, gamma=defg._gamma):
    return np.sqrt((TiTs-1.)*2./(gamma-1.))

def Mach_PiPs(PiPs, gamma=defg._gamma):
    return np.sqrt((PiPs**((gamma-1.)/gamma)-1.)*2./(gamma-1.))

def Velocity_MachTi(Mach, Ti, r=287.1, gamma=defg._gamma):
    return Mach*np.sqrt(gamma*r*Ti/TiTs_Mach(Mach, gamma))



# ===============================================================
# automatic testing

if __name__ == "__main__":
    import doctest
    doctest.testmod()


"""@package Rayleigh
@brief functions to compute Rayleigh (heating) transformation in subsonic/supersonic ducts
"""

import numpy as np

def maxTiratio_Mach(Mach, gamma=1.4):
    """ computes maximum heating ratio parameter for a given Mach number state
    This maximum value makes the flow reach a sonic state
    """
    return 1./Ti_Ticri(Mach, gamma=1.4)

# --- ratio to critical/choking state ---

def Ps_Pscri(Mach, gamma=1.4):
    """ computes Ps/Ps_cri ratio from Mach number
    """
    return (gamma+1)/(1+gamma*np.square(Mach))

def Ts_Tscri(Mach, gamma=1.4):
    """ computes Ts/Ts_cri ratio from Mach number
    """
    m2 = np.square(Mach)
    return m2*np.square(((gamma+1)/(1+gamma*m2)))
    
def Ti_Ticri(Mach, gamma=1.4):
    """ computes Ti/Ti_cri ratio from Mach number
    """
    m2 = np.square(Mach)
    return 2*(gamma+1)*m2 / np.square(1+gamma*m2) * (1+(gamma-1)/2*m2)
    
def Rho_Rhocri(Mach, gamma=1.4):
    """ computes rho/rho_cri ratio from Mach number
    """
    m2 = np.square(Mach)
    return (1+gamma*m2)/((gamma+1)*m2)
    
def Pi_Picri(Mach, gamma=1.4):
    """ computes Pi/Pi_cri ratio from Mach number
    """
    m2 = np.square(Mach)
    return (gamma+1)/(1+gamma*m2)*(2/(gamma+1)*(1+(gamma-1)/2*m2))**(gamma/(gamma-1))

def V_Vcri(Mach, gamma=1.4):
    """ computes V/a_cri ratio from Mach number
    """
    m2 = np.square(Mach)
    return m2*(gamma+1)/(1+gamma*m2)

def NormdS(Mach, gamma=1.4):
    """ computes delta S / Cp up to critical point
    """
    m2 = np.square(Mach)
    return np.log(m2*((gamma+1)/(1+gamma*m2))**((gamma+1)/gamma))

def SubMach_TiTicri(Tiratio, gamma=1.4):
    """ computes Mach number from Ti/Ti_cri ratio (<1)
    """
    alpha = np.sqrt(1.-Tiratio)
    return np.sqrt((1.-alpha)/(alpha*gamma+1))

def SupMach_TiTicri(Tiratio, gamma=1.4):
    """ computes Mach number from Ti/Ti_cri ratio (<1)
    """
    alpha = np.sqrt(1.-Tiratio)
    return np.sqrt((1.+alpha)/(1.-alpha*gamma))




import matplotlib.pyplot as plt

sty_shock = { 'color': 'red',    'linewidth': 3 }
sty_carac = { 'color': 'orange', 'linewidth': 2 }
sty_wall  = { 'color': 'black',  'linewidth': 3 }
sty_flow  = { 'color': 'green',  'linewidth': 3 }
sty_text  = { 'fontsize': 14 }

def set_grid(ax=plt):
	ax.minorticks_on()
	ax.grid(which='major', linestyle='-', alpha=0.8)
	ax.grid(which='minor', linestyle=':', alpha=0.5)

def figure_theta_sigma(**kwargs):
	fig = plt.figure(**kwargs)
	set_grid()
	return fig

def figure_theta_pressure(**kwargs):
	fig = plt.figure(**kwargs)
	set_grid()
	plt.yscale('log')
	return fig




"""
    The ``gasgenerator`` module
    =========================
 
    Provides standart Stagnation to Static Ratios for Ideal gas
 
    :Example:
 
    >>> import aerokit.aero.Isentropic as Is
    >>> Is.TiTs_Mach(1.)
    1.2
    >>> Is.TiTs_Mach(2., gamma=1.6)
    2.2
 
    Available functions
    -------------------
 
    Provides Ti/Ts Pi/Ps ratios from Mach number and reversed functions.
    Specific heat ratio `gamma` is optionnal and can be specified in the functions itself
    or using aerokit.common.defaultgas module
 """

import math
import numpy as np
from aerokit.common import defaultgas as defg # relative import is deprecated by doctest
import aerokit.aero.Isentropic     as Is
import aerokit.engine.gasgenerator as gg

# ===============================================================
# implemented functions

class turbojet_opt(gg.base):
    """
    	Computes Ti/Ts ratio from Mach number

		Long comment
 
		:param Mach:  local Mach number
        :param gamma: specific heat ratio, default from aerokit.common.defaultgas
		:return:      result Ti/Ts ratio
 
 		:Example:

		>>> TiTs_Mach(1.) # with default gamma 1.4
		1.2

		.. seealso:: 
		.. note:: available for scalar or array (numpy) computations
    """	
    def __init__(self, OPR, Tt4, gam_cold=defg._gamma, gam_hot=defg._gamma, r_cold=defg._r, r_hot=defg._r,
                                 xi_inlet=1., xi_nozzle=1., xi_cc=1., eta_cc=1., Pci=42.8e6,
                                 etapolCHP=1., etapolTHP=1., eta_shaft=1.):
        gg.base.__init__(self, OPR, Tt4, gam_cold=gam_cold, gam_hot=gam_hot, r_cold=r_cold, r_hot=r_hot,
                                 xi_inlet=xi_inlet, xi_cc=xi_cc, eta_cc=eta_cc, Pci=Pci,
                                 etapolCHP=etapolCHP, etapolTHP=etapolTHP, eta_shaft=eta_shaft)
        self.xi_nozzle = xi_nozzle


    def update(self):
        gg.base.update(self)
        gh  = self.gam_hot
        cph = gh*self.r_hot/(gh-1.)
        self.Pt9 = np.maximum(self.Pt45 * self.xi_nozzle, self.P0)
        self.M9  = Is.Mach_PiPs(self.Pt9/self.P0, gamma=self.gam_hot)
        self.V9  = Is.Velocity_MachTi(self.M9, self.Tt45, r=self.r_hot, gamma=self.gam_hot)

    def Wsp_kinEn(self):       
        return .5*((1.+self.far)*self.V9**2-self.V0**2)

    def norm_thrust(self):
        return (1.+self.far)*self.V9-self.V0

    def spec_thrust(self):
        return self.norm_thrust()

    def thermal_efficiency(self):
        return self.Wsp_kinEn()/self.Wsp_cc()

    def propulsive_efficiency(self):
        return self.norm_thrust()*self.V0/self.Wsp_kinEn()

    def thermoprop_efficiency(self):
        return self.norm_thrust()*self.V0/self.Wsp_cc()

    def Sfc(self):
        return self.far/self.norm_thrust()

# ===============================================================
# automatic testing

if __name__ == "__main__":
    import doctest
    doctest.testmod()


# -*- coding: utf-8 -*-
"""
Description
-----------
Module containing the definition extended variables according to vtk calculator syntax

Contains
--------

Examples
--------

Notes
-----
"""
from dicovar import dicoVar as dico

# -----------------------------------------------------------------

def expressions_from_rhopv():
    extdico = {}
    # copy dicovar (same as deepcopy)
    for var in dico.keys():
        #print 'copy ',var, dico[var]
        extdico[var] = dico[var]
    # additional variables
    _g     = '1.4'
    _gsgmu = '3.5'
    _gmusd = '0.2'
    _a     = 'sqrt('+_g+'*'+dico['P']+'/'+dico['rho']+')'
    extdico['Pt']   = dico['P']+'*(1+'+_gmusd+'*(mag('+dico['V']+'))^2/('+_g+'*'+dico['P']+'/'+dico['rho']+'))^'+_gsgmu
    extdico['T']    = dico['P']+'/'+dico['rho']+'/287.1'
    extdico['S']    = 'ln('+dico['P']+'/'+dico['rho']+'^'+_g+')'
    extdico['V']    = 'mag('+dico['V']+')'
    extdico['a' ]   = _a
    extdico['Mach'] = 'mag('+dico['V']+')/'+_a
    extdico['Mx']   = 'U_X/'+_a
    extdico['alpha'] = 'atan('+dico['Vy']+'/'+dico['Vx']+')*180/acos(-1)'
    return extdico